/*###################################################################*/
/*##                          GQmpeg 0.4.5                         ##*/
/*##    This software is Copyright (C) 1998, 1999 by John Ellis.   ##*/
/*## This software falls under the GNU Public License. Please read ##*/
/*##              the COPYING file for more information            ##*/
/*###################################################################*/

#include "gqmpeg.h"

static GtkWidget *configwindow;
static GtkAdjustment *buffer_adjustment;
static GtkWidget *buffer_spinbutton;
static GtkAdjustment *sample_adjustment;
static GtkWidget *sample_spinbutton;
static GtkWidget *device_entry;
static GtkWidget *mixer_entry;
static GtkWidget *initial_playlist_entry;
static GtkWidget *initial_directory_entry;
static GtkWidget *default_skin_entry;
static GtkWidget *preset_name_entry[10];
static GtkWidget *preset_file_entry[10];

static gint c_mpg123_buffer_enable;
static gint c_mpg123_buffer;
static gint c_mpg123_downsample;
static gint c_mpg123_custom_sample_size;
static gint c_mpg123_mono;
static gint c_mpg123_8bit;
static gint c_mpg123_device_enable;
static gint c_initial_playlist_mode;
static gint c_initial_directory_enable;
static gint c_show_playlist_on_startup;
static gint c_start_playing_on_startup;
static gint c_obey_mode_in_playlist;
static gint c_DOT_FILES;
static gint c_FILTERING_DISABLE;
static gint c_read_id3_tags;
static gint c_playlist_accounting;
static gint c_smart_window_placement;
static gint c_remember_window_position;
static gint c_mixer_device_id;
static gint c_wm_decorations;
static gint c_play_presets;
static gint c_show_extra_winamp_info;
static gint c_double_size_mode;
static gint c_allow_ipc;

static void allow_ipc_clicked(GtkWidget *w, gpointer data);
static void double_size_mode_clicked(GtkWidget *w, gpointer data);
static void show_extra_winamp_info_clicked(GtkWidget *w, gpointer data);
static void play_presets_clicked(GtkWidget *w, gpointer data);
static void mixer_device_id_cb(GtkWidget *w, gpointer data);
static void set_current_playlist_clicked(GtkWidget *w, gpointer data);
static void set_current_dir_clicked(GtkWidget *w, gpointer data);
static void initial_playlist_empty_selected(GtkWidget *w, gpointer data);
static void initial_playlist_session_selected(GtkWidget *w, gpointer data);
static void initial_playlist_file_selected(GtkWidget *w, gpointer data);
static void initial_directory_enable_clicked(GtkWidget *w, gpointer data);
static void start_playing_clicked(GtkWidget *w, gpointer data);
static void show_playlist_clicked(GtkWidget *w, gpointer data);
static void obey_playlist_clicked(GtkWidget *w, gpointer data);
static void hidden_files_clicked(GtkWidget *w, gpointer data);
static void filter_files_clicked(GtkWidget *w, gpointer data);
static void window_placement_clicked(GtkWidget *w, gpointer data);
static void window_position_clicked(GtkWidget *w, gpointer data);
static void id3_tags_clicked(GtkWidget *w, gpointer data);
static void playlist_accounting_clicked(GtkWidget *w, gpointer data);
static void wm_decorations_clicked(GtkWidget *w, gpointer data);
static void buffer_enable_clicked(GtkWidget *w, gpointer data);
static void downsample_auto_selected(GtkWidget *w, gpointer data);
static void downsample_22_selected(GtkWidget *w, gpointer data);
static void downsample_11_selected(GtkWidget *w, gpointer data);
static void downsample_custom_selected(GtkWidget *w, gpointer data);
static void mono_enable_clicked(GtkWidget *w, gpointer data);
static void enable_8bit_clicked(GtkWidget *w, gpointer data);
static void device_enable_clicked(GtkWidget *w, gpointer data);
static void skin_item_selected(GtkWidget *clist, gint row, gint column,
		GdkEventButton *bevent, gpointer data);
static gint sort_skin_list_cb(void *a, void *b);
static void populate_skin_list(GtkWidget *clist);
static void hide_configuration_window();
static void ok_configuration_window();
static void apply_configuration_window();
static void create_config_window(gint start_tab);

static void allow_ipc_clicked(GtkWidget *w, gpointer data)
{
	c_allow_ipc = GTK_TOGGLE_BUTTON (w)->active;
}

static void double_size_mode_clicked(GtkWidget *w, gpointer data)
{
	c_double_size_mode = GTK_TOGGLE_BUTTON (w)->active;
}

static void show_extra_winamp_info_clicked(GtkWidget *w, gpointer data)
{
	c_show_extra_winamp_info = GTK_TOGGLE_BUTTON (w)->active;
}

static void play_presets_clicked(GtkWidget *w, gpointer data)
{
	c_play_presets = GTK_TOGGLE_BUTTON (w)->active;
}

static void mixer_device_id_cb(GtkWidget *w, gpointer data)
{
	gchar *dev_text = data;
	gint d = get_mixer_device_id(dev_text);
	if (d >= 0) c_mixer_device_id = d;
}

static void set_current_playlist_clicked(GtkWidget *w, gpointer data)
{
	if (!isfile(playlist_pathname)) return;
	gtk_entry_set_text(GTK_ENTRY(initial_playlist_entry),playlist_pathname);
}

static void set_current_dir_clicked(GtkWidget *w, gpointer data)
{
	if (!isdir(current_dir)) return;
	gtk_entry_set_text(GTK_ENTRY(initial_directory_entry),current_dir);
}

static void initial_playlist_empty_selected(GtkWidget *w, gpointer data)
{
	if (GTK_TOGGLE_BUTTON(w)->active)
		c_initial_playlist_mode = PLAYLIST_MODE_EMPTY;
}

static void initial_playlist_session_selected(GtkWidget *w, gpointer data)
{
	if (GTK_TOGGLE_BUTTON(w)->active)
		c_initial_playlist_mode = PLAYLIST_MODE_SESSION;
}

static void initial_playlist_file_selected(GtkWidget *w, gpointer data)
{
	if (GTK_TOGGLE_BUTTON(w)->active)
		c_initial_playlist_mode = PLAYLIST_MODE_FILE;
}

static void initial_directory_enable_clicked(GtkWidget *w, gpointer data)
{
	c_initial_directory_enable = GTK_TOGGLE_BUTTON (w)->active;
}

static void start_playing_clicked(GtkWidget *w, gpointer data)
{
	c_start_playing_on_startup = GTK_TOGGLE_BUTTON (w)->active;
}

static void show_playlist_clicked(GtkWidget *w, gpointer data)
{
	c_show_playlist_on_startup = GTK_TOGGLE_BUTTON (w)->active;
}

static void obey_playlist_clicked(GtkWidget *w, gpointer data)
{
	c_obey_mode_in_playlist = GTK_TOGGLE_BUTTON (w)->active;
}

static void hidden_files_clicked(GtkWidget *w, gpointer data)
{
	c_DOT_FILES = GTK_TOGGLE_BUTTON (w)->active;
}

static void filter_files_clicked(GtkWidget *w, gpointer data)
{
	c_FILTERING_DISABLE = GTK_TOGGLE_BUTTON (w)->active;
}

static void window_placement_clicked(GtkWidget *w, gpointer data)
{
	c_smart_window_placement = GTK_TOGGLE_BUTTON (w)->active;
}

static void window_position_clicked(GtkWidget *w, gpointer data)
{
	c_remember_window_position = GTK_TOGGLE_BUTTON (w)->active;
}

static void id3_tags_clicked(GtkWidget *w, gpointer data)
{
	c_read_id3_tags = GTK_TOGGLE_BUTTON (w)->active;
}

static void playlist_accounting_clicked(GtkWidget *w, gpointer data)
{
	c_playlist_accounting = GTK_TOGGLE_BUTTON (w)->active;
}

static void wm_decorations_clicked(GtkWidget *w, gpointer data)
{
	c_wm_decorations = GTK_TOGGLE_BUTTON (w)->active;
}

static void buffer_enable_clicked(GtkWidget *w, gpointer data)
{
	c_mpg123_buffer_enable = GTK_TOGGLE_BUTTON (w)->active;
}

static void downsample_auto_selected(GtkWidget *w, gpointer data)
{
	if (GTK_TOGGLE_BUTTON(w)->active)
		c_mpg123_downsample = MPG123_DOWNSAMPLE_AUTO;
}

static void downsample_22_selected(GtkWidget *w, gpointer data)
{
	if (GTK_TOGGLE_BUTTON(w)->active)
		c_mpg123_downsample = MPG123_DOWNSAMPLE_22;
}

static void downsample_11_selected(GtkWidget *w, gpointer data)
{
	if (GTK_TOGGLE_BUTTON(w)->active)
		c_mpg123_downsample = MPG123_DOWNSAMPLE_11;
}

static void downsample_custom_selected(GtkWidget *w, gpointer data)
{
	if (GTK_TOGGLE_BUTTON(w)->active)
		c_mpg123_downsample = MPG123_DOWNSAMPLE_CUSTOM;
}

static void mono_enable_clicked(GtkWidget *w, gpointer data)
{
	c_mpg123_mono = GTK_TOGGLE_BUTTON (w)->active;
}

static void enable_8bit_clicked(GtkWidget *w, gpointer data)
{
	c_mpg123_8bit = GTK_TOGGLE_BUTTON (w)->active;
}

static void device_enable_clicked(GtkWidget *w, gpointer data)
{
	c_mpg123_device_enable = GTK_TOGGLE_BUTTON (w)->active;
}

static void skin_item_selected(GtkWidget *clist, gint row, gint column,
		GdkEventButton *bevent, gpointer data)
{
	gchar *text = gtk_clist_get_row_data(GTK_CLIST(clist), row);
	if (text) gtk_entry_set_text(GTK_ENTRY(default_skin_entry),text);
}

static gint sort_skin_list_cb(void *a, void *b)
{
	return strcmp((gchar *)a, (gchar *)b);
}

static void populate_skin_list(GtkWidget *clist)
{
	DIR *dp;
	struct dirent *dir;
	struct stat ent_sbuf;
	gint row;
	gchar *buf[] = { "x", };
	gchar *skinpath;
	GList *skin_list = NULL;
	GList *list;

	/* add default skin */
	buf[0] = "None (default)";
	row = gtk_clist_append(GTK_CLIST(clist), buf);
	gtk_clist_set_row_data(GTK_CLIST(clist), row, "");

	skinpath = g_strconcat(homedir(), "/.gqmpeg/skins", NULL);

	if((dp = opendir(skinpath))==NULL)
		{
		/* dir not found */
		g_free(skinpath);
		return;
		}

	while ((dir = readdir(dp)) != NULL)
		{
		/* skips removed files */
		if (dir->d_ino > 0)
			{
			gchar *name;
			gchar *path;

			name = dir->d_name; 
			path = g_strconcat(skinpath, "/", name, NULL);

			if (stat(path,&ent_sbuf) >= 0 && S_ISDIR(ent_sbuf.st_mode))
				{
				skin_list = g_list_insert_sorted(skin_list,
						g_strdup(name),
						(GCompareFunc) sort_skin_list_cb);
				}
			g_free(path);
			}
		}
        closedir(dp);

	list = skin_list;
	while (list)
		{
		gchar *skindata_file = g_strconcat(skinpath, "/", list->data, "/skindata", NULL);
		if (isfile(skindata_file))
			{
			buf[0] = list->data;
			row = gtk_clist_append(GTK_CLIST(clist),buf);
			gtk_clist_set_row_data_full(GTK_CLIST(clist), row,
					list->data, (GtkDestroyNotify) g_free);
			}
		else
			{
			g_free(skindata_file);
			skindata_file = g_strconcat(skinpath, "/", list->data, "/main.bmp", NULL);
			if (isfile(skindata_file))
				{
				gchar *amp_txt = g_strconcat (list->data, " (amp)", NULL);
				buf[0] = amp_txt;
				row = gtk_clist_append(GTK_CLIST(clist),buf);
				gtk_clist_set_row_data_full(GTK_CLIST(clist), row,
						list->data, (GtkDestroyNotify) g_free);
				g_free(amp_txt);
				}
			else
				{
				gchar *name = list->data;
				if (strcmp(name, ".") != 0 && strcmp(name, "..") != 0)
					{
					printf("Valid skin not found in:%s/%s\n", skinpath, name);
					printf("File named skindata (gqmpeg skins) or main.bmp (winamp skins) not found.\n");
					}
				g_free(list->data);
				}
			}
		g_free(skindata_file);
		list = list->next;
		}

	g_list_free(skin_list);

	g_free(skinpath);
}

void show_about_info()
{
	if (!configwindow)
		create_config_window(6);
		gtk_widget_show(configwindow);
}

void show_configuration_window()
{
	if (configwindow)
		{
		gdk_window_raise(configwindow->window);
		return;
		}

	create_config_window(0);
	gtk_widget_show_all(configwindow);
}

static void hide_configuration_window()
{
	gtk_widget_destroy(configwindow);
	configwindow = NULL;
}

static void ok_configuration_window()
{
	apply_configuration_window();
	hide_configuration_window();
}

static void apply_configuration_window()
{
	gchar *buf = NULL;
	gint i;
	gint skin_changed = FALSE;

	initial_playlist_mode = c_initial_playlist_mode;

	if (initial_playlist_pathname)
		{
		g_free(initial_playlist_pathname);
		initial_playlist_pathname = NULL;
		}
	buf = gtk_entry_get_text(GTK_ENTRY(initial_playlist_entry));
	if (buf&& strlen(buf) > 0) initial_playlist_pathname = g_strdup(buf);

	if ( double_size_mode != c_double_size_mode || 
		show_extra_winamp_info != c_show_extra_winamp_info) skin_changed = TRUE;

	double_size_mode = c_double_size_mode;
	show_extra_winamp_info = c_show_extra_winamp_info;

	buf = gtk_entry_get_text(GTK_ENTRY(default_skin_entry));
	if ( (buf && default_skin && strcmp(buf,default_skin) != 0 && strlen(buf) > 0) ||
		(!default_skin && buf && strlen(buf) > 0) || 
		(buf && strlen(buf) > 0 && skin_changed))
		{
		/* change to the new skin */
		gchar *skin_path = g_strconcat(homedir(), "/.gqmpeg/skins/", buf, NULL);
		change_to_skin(skin_path);
		g_free(skin_path);
		}
	else if ((default_skin && buf && strlen(buf) == 0) || skin_changed)
		{
		change_to_skin(NULL);
		}
	g_free(default_skin);
	default_skin = NULL;
	if (buf && strlen(buf) > 0) default_skin = g_strdup(buf);

	if (mixer_device_id != c_mixer_device_id)
		set_mixer_device(c_mixer_device_id);
	mixer_device_id = c_mixer_device_id;

	if (mixer_command)
		{
		g_free(mixer_command);
		mixer_command = NULL;
		}
	buf = gtk_entry_get_text(GTK_ENTRY(mixer_entry));
	if (buf && strlen(buf) > 0) mixer_command = g_strdup(buf);

	start_playing_on_startup = c_start_playing_on_startup;
	show_playlist_on_startup = c_show_playlist_on_startup;

	obey_mode_in_playlist = c_obey_mode_in_playlist;

	smart_window_placement = c_smart_window_placement;
	remember_window_position = c_remember_window_position;
	wm_decorations = c_wm_decorations;

	play_presets = c_play_presets;

	if (read_id3_tags != c_read_id3_tags)
		{
		read_id3_tags = c_read_id3_tags;
		playlist_update_tags();
		}

	if (playlist_accounting != c_playlist_accounting)
		{
		playlist_accounting = c_playlist_accounting;
		playlist_update_headers();
		show_playlist_length();
		}

	if (DOT_FILES != c_DOT_FILES || FILTERING_DISABLE != c_FILTERING_DISABLE)
		{
		DOT_FILES = c_DOT_FILES;
		FILTERING_DISABLE = c_FILTERING_DISABLE;
		if (playlist_window)
			refresh_dir();
		else
			read_dir();
		}

	initial_directory_enable = c_initial_directory_enable;
	if (initial_directory_path)
		{
		g_free(initial_directory_path);
		initial_directory_path = NULL;
		}
	buf = remove_trailing_slash(gtk_entry_get_text(GTK_ENTRY(initial_directory_entry)));
	if (buf && strlen(buf) > 0) initial_directory_path = g_strdup(buf);
	g_free(buf);

	mpg123_buffer_enable = c_mpg123_buffer_enable;
	mpg123_buffer = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(buffer_spinbutton));
	mpg123_downsample = c_mpg123_downsample;
	mpg123_custom_sample_size = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(sample_spinbutton));
	mpg123_mono = c_mpg123_mono;
	mpg123_8bit = c_mpg123_8bit;
	mpg123_device_enable = c_mpg123_device_enable;

	if (mpg123_device)
		{
		g_free(mpg123_device);
		mpg123_device = NULL;
		}
	buf = gtk_entry_get_text(GTK_ENTRY(device_entry));
	if (buf && strlen(buf) > 0) mpg123_device = g_strdup(buf);

	if (allow_ipc != c_allow_ipc)
		{
		allow_ipc = c_allow_ipc;
		if (allow_ipc)
			ipc_on();
		else
			ipc_off();
		}

	for (i=0; i<10; i++)
		{
		if (preset_name[i]) g_free(preset_name[i]);
		if (preset_file[i]) g_free(preset_file[i]);

		buf = gtk_entry_get_text(GTK_ENTRY(preset_name_entry[i]));
		if (strlen(buf) > 0)
			preset_name[i] = g_strdup(buf);
		else
			preset_name[i] = NULL;

		buf = gtk_entry_get_text(GTK_ENTRY(preset_file_entry[i]));
		if (strlen(buf) > 0)
			preset_file[i] = g_strdup(buf);
		else
			preset_file[i] = NULL;
		}

	create_preset_menu();
	gdk_window_set_decorations (mainwindow->window, wm_decorations);
}

static void create_config_window(gint start_tab)
{
	GtkWidget *configvbox;
	GtkWidget *configtabs;
	GtkWidget *radiobutton;
	GtkWidget *button;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *vbox1;
	GtkWidget *label;
	GtkWidget *frame;
#ifdef GTK_HAVE_FEATURES_1_1_4
	GtkWidget *scrolled;
#endif
	GtkWidget *skin_clist;
	GtkWidget *table;
	GtkWidget *sep;

        GtkWidget *omenu;
        GtkWidget *menu;
        GtkWidget *item;
	GList *devices;
	GList *list;
	gint lh, cnt, i;

	GtkWidget *about_icon;
	GdkPixmap *about_pixmap;
	GdkBitmap *about_mask;
	GtkStyle *style;

	gchar *skin_title [] = { "Available skins:", };
	gchar *skin_label_text;
	gchar infostr[255];

	configwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_signal_connect (GTK_OBJECT (configwindow), "delete_event",(GtkSignalFunc) hide_configuration_window, NULL);
	gtk_window_set_policy (GTK_WINDOW (configwindow), FALSE, FALSE, TRUE);
	gtk_window_set_title (GTK_WINDOW (configwindow), "GQmpeg configuration");
	gtk_container_border_width (GTK_CONTAINER (configwindow), 5);

	configvbox = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(configwindow),configvbox);
	gtk_widget_show(configvbox);

	hbox = gtk_hbox_new (TRUE, 0);
	gtk_box_pack_end(GTK_BOX(configvbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	button = gtk_button_new_with_label("Ok");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) ok_configuration_window, NULL);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 20);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Apply");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) apply_configuration_window, NULL);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 20);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Cancel");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) hide_configuration_window, NULL);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 20);
	gtk_widget_show(button);

	configtabs = gtk_notebook_new();
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK(configtabs), GTK_POS_TOP);
	gtk_box_pack_start (GTK_BOX(configvbox), configtabs, TRUE, TRUE, 0);

	/* general tab */

	frame = gtk_frame_new(NULL);
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_widget_show(frame);
	label = gtk_label_new("General");
	gtk_notebook_append_page (GTK_NOTEBOOK (configtabs), frame, label);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (vbox), 5);
	gtk_container_add (GTK_CONTAINER(frame),vbox);
	gtk_widget_show(vbox);
	
	frame = gtk_frame_new("Song Information");
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER(frame),vbox1);
	gtk_widget_show(vbox1);

	c_playlist_accounting = playlist_accounting;
	button = gtk_check_button_new_with_label ("Read file headers (enables playlist length calculation)");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_playlist_accounting);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) playlist_accounting_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	c_read_id3_tags = read_id3_tags;
	button = gtk_check_button_new_with_label ("Use ID3 tags when available");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_read_id3_tags);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) id3_tags_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	frame = gtk_frame_new("Window");
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER(frame),vbox1);
	gtk_widget_show(vbox1);

	c_smart_window_placement = smart_window_placement;
	button = gtk_check_button_new_with_label ("Smart window placement");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_smart_window_placement);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) window_placement_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	c_remember_window_position = remember_window_position;
	button = gtk_check_button_new_with_label ("Remember window position");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_remember_window_position);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) window_position_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	c_wm_decorations = wm_decorations;
	button = gtk_check_button_new_with_label ("Enable window manager decorations");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_wm_decorations);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) wm_decorations_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	c_double_size_mode = double_size_mode;
	button = gtk_check_button_new_with_label ("Double sized window");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_double_size_mode);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) double_size_mode_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	frame = gtk_frame_new("Mixer");
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER(frame),vbox1);
	gtk_widget_show(vbox1);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	label = gtk_label_new("External Mixer:");
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	mixer_entry = gtk_entry_new_with_max_length(511);
	if (mixer_command)
		gtk_entry_set_text(GTK_ENTRY(mixer_entry),mixer_command);
	tab_completion_add_to_entry(mixer_entry, NULL, NULL);
	gtk_box_pack_start(GTK_BOX(hbox),mixer_entry,TRUE,TRUE,5);
	gtk_widget_show(mixer_entry);

	c_mixer_device_id = mixer_device_id;
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	label = gtk_label_new("Volume controls device:");
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

        omenu = gtk_option_menu_new ();
        menu = gtk_menu_new ();

	devices = get_mixer_device_list();

	list = devices;

	cnt = 0;
	while(list)
		{
		gchar *dev_text = list->data;
	        item = gtk_menu_item_new_with_label(dev_text);
	        gtk_signal_connect (GTK_OBJECT (item), "activate", (GtkSignalFunc) mixer_device_id_cb, dev_text);
        	gtk_menu_append (GTK_MENU (menu), item);
		if (get_mixer_device_id(dev_text) == mixer_device_id) lh = cnt;
		cnt++;
		list = list->next;
		}

	g_list_free(devices);

	gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
        gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), lh);
        gtk_box_pack_start( GTK_BOX(hbox), omenu, TRUE, TRUE, 5);
        gtk_widget_show (omenu);

	c_allow_ipc = allow_ipc;
	button = gtk_check_button_new_with_label ("Allow command parsing through ~/.gqmpeg/command");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_allow_ipc);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) allow_ipc_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	/* Startup tab */

	frame = gtk_frame_new(NULL);
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_widget_show(frame);
	label = gtk_label_new("Startup");
	gtk_notebook_append_page (GTK_NOTEBOOK (configtabs), frame, label);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (vbox), 5);
	gtk_container_add (GTK_CONTAINER(frame),vbox);
	gtk_widget_show(vbox);
	
	frame = gtk_frame_new("Initial playlist");
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER(frame),vbox1);
	gtk_widget_show(vbox1);

	c_initial_playlist_mode = initial_playlist_mode;
	radiobutton = gtk_radio_button_new_with_label (NULL,"Empty");
	if (c_initial_playlist_mode == PLAYLIST_MODE_EMPTY) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(radiobutton), 1);
	gtk_signal_connect (GTK_OBJECT(radiobutton),"clicked",(GtkSignalFunc) initial_playlist_empty_selected, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), radiobutton, FALSE, FALSE, 0);
	gtk_widget_show(radiobutton);

	button = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton)),"Save playlist between sessions");
	if (c_initial_playlist_mode == PLAYLIST_MODE_SESSION) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), 1);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) initial_playlist_session_selected, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	button = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton)),"Open specific playlist:");
	if (c_initial_playlist_mode == PLAYLIST_MODE_FILE) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), 1);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) initial_playlist_file_selected, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	initial_playlist_entry = gtk_entry_new_with_max_length(511);
	if (initial_playlist_pathname)
		gtk_entry_set_text(GTK_ENTRY(initial_playlist_entry),initial_playlist_pathname);
	tab_completion_add_to_entry(initial_playlist_entry, NULL, NULL);
	gtk_box_pack_start(GTK_BOX(hbox),initial_playlist_entry,TRUE,TRUE,0);
	gtk_widget_show(initial_playlist_entry);

	button = gtk_button_new_with_label("Use current");
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc)set_current_playlist_clicked, NULL);
	gtk_box_pack_end(GTK_BOX(hbox),button,FALSE,FALSE,0);
	gtk_widget_show(button);

	frame = gtk_frame_new("When program starts:");
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER(frame),vbox1);
	gtk_widget_show(vbox1);

	c_start_playing_on_startup = start_playing_on_startup;
	button = gtk_check_button_new_with_label ("Start playing");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_start_playing_on_startup);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) start_playing_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	c_show_playlist_on_startup = show_playlist_on_startup;
	button = gtk_check_button_new_with_label ("Show playlist window");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_show_playlist_on_startup);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) show_playlist_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	c_initial_directory_enable = initial_directory_enable;
	button = gtk_check_button_new_with_label ("Change to this directory:");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_initial_directory_enable);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) initial_directory_enable_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	initial_directory_entry = gtk_entry_new_with_max_length(255);
	if (initial_directory_path)
		gtk_entry_set_text(GTK_ENTRY(initial_directory_entry),initial_directory_path);
	tab_completion_add_to_entry(initial_directory_entry, NULL, NULL);
	gtk_box_pack_start(GTK_BOX(hbox),initial_directory_entry,TRUE,TRUE,0);
	gtk_widget_show(initial_directory_entry);

	button = gtk_button_new_with_label("Use current");
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc)set_current_dir_clicked, NULL);
	gtk_box_pack_end(GTK_BOX(hbox),button,FALSE,FALSE,0);
	gtk_widget_show(button);

	/* output tab */

	frame = gtk_frame_new(NULL);
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_widget_show(frame);
	label = gtk_label_new("Output");
	gtk_notebook_append_page (GTK_NOTEBOOK (configtabs), frame, label);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (vbox), 5);
	gtk_container_add (GTK_CONTAINER(frame),vbox);
	gtk_widget_show(vbox);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	c_mpg123_buffer_enable = mpg123_buffer_enable;
	button = gtk_check_button_new_with_label ("Enable output buffer of size(Kbytes):");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_mpg123_buffer_enable);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) buffer_enable_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	c_mpg123_buffer = mpg123_buffer;
	buffer_adjustment = (GtkAdjustment *)gtk_adjustment_new ((float)c_mpg123_buffer,0.0,8192.0,8.0,8.0,8.0);
	buffer_spinbutton = gtk_spin_button_new (buffer_adjustment, (float)8, 0);
	gtk_widget_set_usize(buffer_spinbutton,60,-1);
	gtk_box_pack_start(GTK_BOX(hbox), buffer_spinbutton, FALSE, FALSE, 0);
	gtk_widget_show(buffer_spinbutton);

	/* downsampling */

	frame = gtk_frame_new("Downsampling");
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER(frame),vbox1);
	gtk_widget_show(vbox1);

	c_mpg123_downsample = mpg123_downsample;
	radiobutton = gtk_radio_button_new_with_label (NULL,"Auto or(1:1)");
	if (c_mpg123_downsample == MPG123_DOWNSAMPLE_AUTO) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(radiobutton), 1);
	gtk_signal_connect (GTK_OBJECT(radiobutton),"clicked",(GtkSignalFunc) downsample_auto_selected, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), radiobutton, FALSE, FALSE, 0);
	gtk_widget_show(radiobutton);

	button = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton)),"22Khz (2:1)");
	if (c_mpg123_downsample == MPG123_DOWNSAMPLE_22) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), 1);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) downsample_22_selected, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	button = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton)),"11Khz (4:1)");
	if (c_mpg123_downsample == MPG123_DOWNSAMPLE_11) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), 1);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) downsample_11_selected, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	button = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton)),"Custom (bits/sec):");
	if (c_mpg123_downsample == MPG123_DOWNSAMPLE_CUSTOM) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), 1);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) downsample_custom_selected, NULL);
	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	c_mpg123_custom_sample_size = mpg123_custom_sample_size;
	sample_adjustment = (GtkAdjustment *)gtk_adjustment_new ((float)c_mpg123_custom_sample_size,8000.0,49152.0,8.0,8.0,8.0);
	sample_spinbutton = gtk_spin_button_new (sample_adjustment, (float)8, 0);
	gtk_widget_set_usize(sample_spinbutton,60,-1);
	gtk_box_pack_start(GTK_BOX(hbox), sample_spinbutton, FALSE, FALSE, 0);
	gtk_widget_show(sample_spinbutton);

	c_mpg123_mono = mpg123_mono;
	button = gtk_check_button_new_with_label ("Mix both channels into mono output");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_mpg123_mono);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) mono_enable_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	c_mpg123_8bit = mpg123_8bit;
	button = gtk_check_button_new_with_label ("8-bit output");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_mpg123_8bit);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) enable_8bit_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	c_mpg123_device_enable = mpg123_device_enable;
	button = gtk_check_button_new_with_label ("Specific output device:");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_mpg123_device_enable);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) device_enable_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	device_entry = gtk_entry_new_with_max_length(128);
	if (mpg123_device)
		gtk_entry_set_text(GTK_ENTRY(device_entry),mpg123_device);
	gtk_box_pack_start(GTK_BOX(hbox),device_entry,TRUE,FALSE,0);
	gtk_widget_show(device_entry);

	label = gtk_label_new("Output options do not take effect\n until the next song.");
	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
	gtk_widget_show (label);

	/* playlist tab */

	frame = gtk_frame_new(NULL);
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_widget_show(frame);
	label = gtk_label_new("Playlist");
	gtk_notebook_append_page (GTK_NOTEBOOK (configtabs), frame, label);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (vbox), 5);
	gtk_container_add (GTK_CONTAINER(frame),vbox);
	gtk_widget_show(vbox);

	c_obey_mode_in_playlist = obey_mode_in_playlist;
	button = gtk_check_button_new_with_label ("Obey shuffle / repeat if saved in playlist");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_obey_mode_in_playlist);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) obey_playlist_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	c_DOT_FILES = DOT_FILES;
	button = gtk_check_button_new_with_label ("Show hidden files and directories in playlist window");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_DOT_FILES);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) hidden_files_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	c_FILTERING_DISABLE = FILTERING_DISABLE;
	button = gtk_check_button_new_with_label ("Show all files in the playlist window");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_FILTERING_DISABLE);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) filter_files_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	/* preset tab */

	frame = gtk_frame_new(NULL);
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_widget_show(frame);
	label = gtk_label_new("Presets");
	gtk_notebook_append_page (GTK_NOTEBOOK (configtabs), frame, label);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (vbox), 5);
	gtk_container_add (GTK_CONTAINER(frame),vbox);
	gtk_widget_show(vbox);

	table=gtk_table_new(3,11,FALSE);
	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
        gtk_widget_show(table);

	label = gtk_label_new("#");
        gtk_table_attach_defaults(GTK_TABLE (table),label, 0, 1, 0, 1);
        gtk_widget_show(label);
        label = gtk_label_new("Preset name");
        gtk_table_attach_defaults(GTK_TABLE (table),label, 1, 2, 0, 1);
        gtk_widget_show(label);
        label = gtk_label_new("Playlist file");
        gtk_table_attach_defaults(GTK_TABLE (table),label, 2, 3, 0, 1);
        gtk_widget_show(label);

	for (i=0; i<10; i++)
		{
		gchar s[16];
		hbox = gtk_hbox_new (FALSE, 0);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
		gtk_widget_show(hbox);

		sprintf(s,"%d",i + 1);
                label = gtk_label_new(s);
		gtk_table_attach_defaults(GTK_TABLE (table),label,0,1,i+1,i+2);
		gtk_widget_show(label);

		preset_name_entry[i] = gtk_entry_new_with_max_length(32);
                gtk_widget_set_usize(preset_name_entry[i],80,-1);
		if (preset_name[i]) gtk_entry_set_text(GTK_ENTRY(preset_name_entry[i]), preset_name[i]);
		gtk_table_attach_defaults(GTK_TABLE (table),preset_name_entry[i],1,2,i+1,i+2);
                gtk_widget_show(preset_name_entry[i]);

		preset_file_entry[i] = gtk_entry_new();
		if (preset_file[i]) gtk_entry_set_text(GTK_ENTRY(preset_file_entry[i]), preset_file[i]);
		tab_completion_add_to_entry(preset_file_entry[i], NULL, NULL);
		gtk_table_attach_defaults(GTK_TABLE (table),preset_file_entry[i],2,3,i+1,i+2);
                gtk_widget_show(preset_file_entry[i]);
		}

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	c_play_presets = play_presets;
	button = gtk_check_button_new_with_label ("Play presets immediately");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_play_presets);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) play_presets_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	/* skin tab */

	frame = gtk_frame_new(NULL);
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_widget_show(frame);
	label = gtk_label_new("Skins");
	gtk_notebook_append_page (GTK_NOTEBOOK (configtabs), frame, label);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (vbox), 5);
	gtk_container_add (GTK_CONTAINER(frame),vbox);
	gtk_widget_show(vbox);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	label = gtk_label_new("Skin:");
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
	gtk_widget_show (label);

	default_skin_entry = gtk_entry_new_with_max_length(511);
	if (default_skin)
		gtk_entry_set_text(GTK_ENTRY(default_skin_entry),default_skin);
	gtk_box_pack_start(GTK_BOX(hbox),default_skin_entry,TRUE,TRUE,0);
	gtk_widget_show(default_skin_entry);

#ifdef GTK_HAVE_FEATURES_1_1_4
	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(vbox),scrolled,TRUE,TRUE,0);
	gtk_widget_show(scrolled);
#endif

	/* skin list */
	skin_clist=gtk_clist_new_with_titles (1, skin_title);
	gtk_clist_column_titles_passive (GTK_CLIST (skin_clist)); 
	gtk_signal_connect (GTK_OBJECT (skin_clist), "select_row",(GtkSignalFunc) skin_item_selected, NULL);
#ifdef GTK_HAVE_FEATURES_1_1_4
	gtk_container_add (GTK_CONTAINER (scrolled), skin_clist);
#else
	gtk_clist_set_policy (GTK_CLIST (skin_clist), GTK_POLICY_ALWAYS, GTK_POLICY_AUTOMATIC); 
	gtk_box_pack_start (GTK_BOX (vbox), skin_clist, TRUE, TRUE, 0);
#endif
	gtk_widget_show (skin_clist);

	populate_skin_list(skin_clist);

	skin_label_text = g_strconcat(homedir(), "/.gqmpeg/skins", NULL);

	label = gtk_label_new(skin_label_text);
	gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	g_free(skin_label_text);

	c_show_extra_winamp_info = show_extra_winamp_info;
	button = gtk_check_button_new_with_label ("Display cpu and frame information in amp skins");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), c_show_extra_winamp_info);
	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) show_extra_winamp_info_clicked, NULL);
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	sep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 0);
	gtk_widget_show(sep);

	label = gtk_label_new("Please place custom skins in:");
	gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	/* about tab */

	frame = gtk_frame_new(NULL);
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_widget_show(frame);
	label = gtk_label_new("About");
	gtk_notebook_append_page (GTK_NOTEBOOK (configtabs), frame, label);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER(frame),vbox);
	gtk_widget_show(vbox);

	label = gtk_label_new("");
	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
	gtk_widget_show (label);

	gtk_widget_realize(configwindow);

	style = gtk_widget_get_style(configwindow);
	about_pixmap = gdk_pixmap_create_from_xpm_d(configwindow->window, &about_mask,
		&style->bg[GTK_STATE_NORMAL], (gchar **)gqmpeg_icon_xpm);
	about_icon = gtk_pixmap_new(about_pixmap, about_mask);
	gtk_box_pack_start(GTK_BOX(vbox), about_icon, FALSE, FALSE, 0);
	gtk_widget_show(about_icon);

	sprintf(infostr,"GQmpeg %d.%d.%d\nA frontend to mpg123\n\nCopyright (c) 1998, 1999 by John Ellis\nwww.geocities.com/SiliconValley/Haven/5235\ngqview@geocities.com\n\nReleased under the GNU Public License",GQMPEG_VERSION_MAJOR,GQMPEG_VERSION_MINOR,GQMPEG_VERSION_REV);
	label = gtk_label_new(infostr);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	label = gtk_label_new("");
	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
	gtk_widget_show (label);

	gtk_notebook_set_page (GTK_NOTEBOOK(configtabs), start_tab);
	gtk_widget_show(configtabs);

}


