/*###################################################################*/
/*##                         GQmpeg 0.4.5                          ##*/
/*##    This software is Copyright (C) 1998, 1999 by John Ellis.   ##*/
/*## This software falls under the GNU Public License. Please read ##*/
/*##              the COPYING file for more information            ##*/
/*###################################################################*/

#include "gqmpeg.h"

void update_slider_display()
{
	float pos;

	if (frames_remaining == 0 && frames == 0)
		pos = 0;
	else
		pos = (float)frames / (frames_remaining + frames);
	draw_slider(skin->position, pos, FALSE, TRUE, FALSE, FALSE, FALSE);
}

void update_volume_display()
{
	if (skin->volume)
		draw_slider(skin->volume, ((float)get_volume() / 100 ), FALSE, TRUE, FALSE, FALSE, FALSE);
	else if (skin->volume_item)
		{
		gint section = (float)get_volume() / 100 * (skin->volume_item->sections - 1);
		draw_item(skin->volume_item, section);
		}
}

void update_balance_display()
{
	if (skin->balance)
		draw_slider(skin->balance, ((float)get_balance() / 100 ), FALSE, TRUE, FALSE, FALSE, FALSE);
	else if (skin->balance_item)
		{
		gint section = (float)get_balance() / 100 * (skin->balance_item->sections - 1);
		draw_item(skin->balance_item, section);
		}
}

void update_shuffle_display()
{
	draw_item (skin->item_shuffle, shuffle_mode);
	change_button_light(skin->shuffle, shuffle_mode);
}

void update_repeat_display()
{
	draw_item (skin->item_repeat, repeat_mode);
	change_button_light(skin->repeat, repeat_mode);
}

void update_status_display()
{
	if (status == STATUS_STOP)
		{
		change_button_light(skin->play, FALSE);
		change_button_light(skin->stop, TRUE);
		change_button_light(skin->pause, FALSE);
		draw_item (skin->status, 0);
		}
	if (status == STATUS_PAUSE)
		{
		change_button_light(skin->play, FALSE);
		change_button_light(skin->stop, FALSE);
		change_button_light(skin->pause, TRUE);
		draw_item (skin->status, 1);
		}
	if (status == STATUS_PLAY)
		{
		change_button_light(skin->play, TRUE);
		change_button_light(skin->stop, FALSE);
		change_button_light(skin->pause, FALSE);
		draw_item (skin->status, 2);
		}
}

void update_frame_count_display(gint n)
{
	draw_number (skin->frame, n);
}

void update_cpu_display(gfloat percentf)
{
	draw_number (skin->cpu, (gint)percentf);
}

void update_time_count(gint count, gint remain)
{
	gint n;
	gint m,s;

	if (show_remaining_time)
		n = remain;
	else
		n = count;

	m = n / 60;
	s = n - (m * 60);
	draw_number (skin->min, m);
	draw_number (skin->sec, s);
}

void update_song_number(gint n)
{
	draw_number (skin->song_current, n + 1);
}

void update_song_count(gint n)
{
	draw_number (skin->song_total, n);
	show_playlist_length();
}

void update_io_information()
{
	draw_number (skin->output_bits, output_bits);
	draw_number (skin->output_hz, output_hz / 1000);
	draw_number (skin->input_bits, mpeg_bitrate);
	draw_number (skin->input_hz, mpeg_hz / 1000);

	draw_item (skin->stereo, output_channels);
	draw_item (skin->mpegver, mpeg_version);
	draw_item (skin->layer, mpeg_layer);
	draw_item (skin->mode, mpeg_mode);
}

void redraw_display()
{
	redraw_skin();
}

void redraw_name(gchar *t)
{
	draw_font(skin->title, t);
}

void update_all_display_info()
{
	gint i;

	draw_button(skin->prev, FALSE, FALSE, TRUE);
	draw_button(skin->pause, FALSE, FALSE, TRUE);
	draw_button(skin->play, FALSE, FALSE, TRUE);
	draw_button(skin->stop, FALSE, FALSE, TRUE);
	draw_button(skin->next, FALSE, FALSE, TRUE);
	draw_button(skin->ff, FALSE, FALSE, TRUE);
	draw_button(skin->rw, FALSE, FALSE, TRUE);
	draw_button(skin->shuffle, FALSE, FALSE, TRUE);
	draw_button(skin->repeat, FALSE, FALSE, TRUE);
	draw_button(skin->playlist, FALSE, FALSE, TRUE);
	draw_button(skin->config, FALSE, FALSE, TRUE);
	draw_button(skin->exit, FALSE, FALSE, TRUE);
	draw_button(skin->iconify, FALSE, FALSE, TRUE);
	draw_button(skin->mixer, FALSE, FALSE, TRUE);
	draw_button(skin->skin_mode, FALSE, FALSE, TRUE);

	update_shuffle_display();
	update_repeat_display();
	update_song_number(playlist_song);
	update_song_count(playlist_count);
	update_io_information();
	update_cpu_display(0);
	if (status == STATUS_PLAY)
		{
		update_time_count(seconds, seconds_remaining);
		update_frame_count_display(frames);
		change_button_light(skin->play,  TRUE);
		change_button_light(skin->stop,  FALSE);
		change_button_light(skin->pause, FALSE);
		draw_item (skin->status, 2);
		draw_slider(skin->position, 0, FALSE, FALSE, FALSE, FALSE, TRUE);
		}
	else if (status == STATUS_PAUSE)
		{
		update_time_count(seconds, seconds_remaining);
		update_frame_count_display(frames);
		change_button_light(skin->play,  FALSE);
		change_button_light(skin->stop,  FALSE);
		change_button_light(skin->pause, TRUE);
		draw_item (skin->status, 1);
		draw_slider(skin->position, 0, FALSE, FALSE, FALSE, FALSE, TRUE);
		}
	else
		{
		update_time_count(0, 0);
		update_frame_count_display(0);
		change_button_light(skin->play,  FALSE);
		change_button_light(skin->stop,  TRUE);
		change_button_light(skin->pause, FALSE);
		draw_item (skin->status, 0);
		draw_slider(skin->position, 0, FALSE, TRUE, FALSE, FALSE, TRUE);
		}
	if (skin->volume)
		draw_slider(skin->volume, ((float)get_volume() / 100 ), FALSE, TRUE, FALSE, FALSE, TRUE);
	else if (skin->volume_item)
		{
		gint section = (float)get_volume() / 100 * (skin->volume_item->sections - 1);
		draw_item(skin->volume_item, section);
		}
	draw_button(skin->volume_up, FALSE, FALSE, TRUE);
	draw_button(skin->volume_dn, FALSE, FALSE, TRUE);

	if (skin->balance)
		draw_slider(skin->balance, ((float)get_balance() / 100 ), FALSE, TRUE, FALSE, FALSE, TRUE);
	else if (skin->balance_item)
		{
		gint section = (float)get_balance() / 100 * (skin->balance_item->sections - 1);
		draw_item(skin->balance_item, section);
		}
	draw_button(skin->balance_left, FALSE, FALSE, TRUE);
	draw_button(skin->balance_right, FALSE, FALSE, TRUE);

	for (i=0; i<10; i++)
		{
		draw_button(skin->preset[i], FALSE, FALSE, TRUE);
		}

	redraw_name(get_song_title(playlist_song));
}

