/*###################################################################*/
/*##                          GQmpeg 0.4.5                         ##*/
/*##       This software is Copyright (C) 1999 by John Ellis.      ##*/
/*## This software falls under the GNU Public License. Please read ##*/
/*##              the COPYING file for more information            ##*/
/*###################################################################*/

#include <pwd.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <dirent.h>

/* includes needed for Sun Solaris */
#ifdef sun
  #include <sys/procfs.h>
  #include <fcntl.h>
#endif

/* includes needed for SGI IRIX 5.3 and 6.5 */
#ifdef SGI
  #include <sys/signal.h>
  #include <sys/fault.h>
  #include <sys/syscall.h>
  #include <sys/procfs.h>
  #include <fcntl.h>
#endif

#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gdk_imlib.h>

	/* definitions */
#define		GQMPEG_VERSION_MAJOR 0
#define		GQMPEG_VERSION_MINOR 4
#define		GQMPEG_VERSION_REV 5
#define		list_width 180
#define		list_height 180
#define		playlist_window_width 500
#define		playlist_window_height 400
#define		foreground_color 0
#define		background_color 255

#define		STATUS_STOP 0
#define		STATUS_PAUSE 1
#define		STATUS_PLAY 2
#define		STATUS_NEXT 3

#define		MPG123_DOWNSAMPLE_AUTO 0
#define		MPG123_DOWNSAMPLE_22 1
#define		MPG123_DOWNSAMPLE_11 2
#define		MPG123_DOWNSAMPLE_CUSTOM 3

#define		PLAYLIST_MODE_EMPTY 0
#define		PLAYLIST_MODE_SESSION 1
#define		PLAYLIST_MODE_FILE 2

#define		TYPE_MPEG_FILE 0
#define		TYPE_MPEG_SHOUTCAST 1

typedef struct _ButtonData ButtonData;
struct _ButtonData
{
        GdkPixmap *pixmap;
	gint sections;
	gint has_light;
	gint has_prelight;
	gint pushed;
	gint lit;
	gint prelit;
	gint width;
	gint height;
	gint x;
	gint y;
	void (*press_func)();
	void (*release_func)();
	void (*click_func)();
};

typedef struct _ItemData ItemData;
struct _ItemData
{
        GdkPixmap *pixmap;
	gint sections;
	gint width;
	gint height;
	gint x;
	gint y;
};

typedef struct _FontData FontData;
struct _FontData
{
        GdkPixmap *pixmap;
	gint length;
	gint char_width;
	gint char_height;
	gint width;
	gint height;
	gint x;
	gint y;
};

typedef struct _DigitData DigitData;
struct _DigitData
{
        GdkPixmap *pixmap;
	gint width;
	gint height;
};

typedef struct _NumberData NumberData;
struct _NumberData
{
	DigitData *digits;
	gint x;
	gint y;
	gint length;
	gint zeros;
};

typedef struct _SliderData SliderData;
struct _SliderData
{
	GdkPixmap *pixmap;
	gint x;
	gint y;
	gint width;
	gint height;
	gint verticle;
	gint reversed;
	gint handle_width;
	gint handle_height;
	gint position;
	gint has_prelight;
	gint pushed;
	gint prelit;
	void (*press_func)(float);
	void (*release_func)(float);
	void (*move_func)(float);
	ItemData *item;
};

typedef struct _SkinData SkinData;
struct _SkinData
{
	gint width;
	gint height;
	GdkPixmap *background;
	GdkBitmap *mask;
	FontData *title;
	ItemData *stereo;
	ItemData *item_shuffle;
	ItemData *item_repeat;
	ItemData *mpegver;
	ItemData *layer;
	ItemData *mode;
	ItemData *status;
	NumberData *min;
	NumberData *sec;
	NumberData *song_current;
	NumberData *song_total;
	NumberData *input_bits;
	NumberData *input_hz;
	NumberData *output_bits;
	NumberData *output_hz;
	NumberData *frame;
	NumberData *cpu;
	DigitData *dig_small;
	DigitData *dig_large;
	ButtonData *play;
	ButtonData *stop;
	ButtonData *pause;
	ButtonData *next;
	ButtonData *prev;
	ButtonData *ff;
	ButtonData *rw;
	ButtonData *shuffle;
	ButtonData *repeat;
	ButtonData *playlist;
	ButtonData *config;
	ButtonData *iconify;
	ButtonData *exit;
	ButtonData *mixer;
	ButtonData *skin_mode;
	SliderData *position;
	SliderData *volume;
	ItemData *volume_item;
	ButtonData *volume_up;
	ButtonData *volume_dn;
	SliderData *balance;
	ItemData *balance_item;
	ButtonData *balance_left;
	ButtonData *balance_right;
	ButtonData *preset[10];
};

typedef struct _Mpg_Data Mpg_Data;
struct _Mpg_Data
{
	gint version;
	gint layer;
	gint error_protection;
	gint bit_rate;
	gint sample_rate;
	gint padding;
	gint extension;
	gint channel_mode;
	gint mode_extension;
	gint copyright;
	gint original;
	gint emphasis;

	gchar *version_text;
	gchar *version_number_text;
	gchar *layer_text;
	gchar *mode_text;

	gint stereo;			/* if true, 2 or more channels */
	gint file_length;		/* length of file in bytes */
	gint length;			/* length in seconds */
};

typedef struct _ID3_Data ID3_Data;
struct _ID3_Data
{
	gchar *title;
	gchar *artist;
	gchar *album;
	gchar *year;
	gchar *comment;
	gint genre;
	gchar *genre_description;
};

typedef struct _Song_Data Song_Data;
struct _Song_Data
{
	gchar *path;
	Mpg_Data *hdr;
	ID3_Data *tag;
};

	/* main window widgets */
extern GtkWidget *mainwindow;
extern GtkWidget *display_area;
extern GtkWidget *playlist_window;

extern GtkWidget *main_menu;
extern GtkWidget *dir_menu;
extern GtkWidget *file_menu;
extern GtkWidget *file_menu_list;
extern GtkWidget *playlist_menu;

	/* variables */

extern SkinData *skin;
extern SkinData *skin_normal;
extern SkinData *skin_small;

	/* file lists */
extern GList *filelist;
extern gint filelist_count;
extern GList *dirlist;
extern gint dirlist_count;

	/* filtering */
extern gchar filter[];
extern gint FILTERING_DISABLE;
extern gint DOT_FILES;
extern gint FILTER_INCLUDE_MP3;
extern gint FILTER_INCLUDE_GQMPEG;

	/* playlist info */
extern GList *playlist;
extern gint playlist_count;
extern gint playlist_length; /* length in seconds */
extern gint playlist_pos;
extern gint playlist_song;
extern gchar *playlist_pathname;
extern gchar *playlist_filename;
extern gint playlist_changed;

	/* song info */
extern gchar *song_filename;
extern gchar *song_path;
extern gint frames;
extern gint frames_remaining;
extern gint seconds;
extern gint seconds_remaining;
extern gint mpeg_version;
extern gint mpeg_layer;
extern gint mpeg_bitrate;
extern gint mpeg_mode;
extern gint mpeg_channels;
extern gint mpeg_hz;
extern gint output_channels;
extern gint output_hz;
extern gint output_bits;
extern gint output_conversion;

	/* ID3 tags */
extern gchar *id3_title;
extern gchar *id3_artist;
extern gchar *id3_album;
extern gchar *id3_comment;
extern gchar *id3_year;
extern gint id3_genre;

	/* mpg123 options */
extern gint mpg123_buffer_enable;
extern gint mpg123_buffer;
extern gint mpg123_downsample;
extern gint mpg123_custom_sample_size;
extern gint mpg123_mono;
extern gint mpg123_8bit;
extern gint mpg123_device_enable;
extern gchar *mpg123_device;

	/* options */
extern gint initial_playlist_mode;
extern gchar *initial_playlist_pathname;
extern gint initial_playlist_position;
extern gint initial_directory_enable;
extern gchar *initial_directory_path;
extern gint show_playlist_on_startup;
extern gint start_playing_on_startup;
extern gint smart_window_placement;
extern gint read_id3_tags;
extern gint playlist_accounting;
extern gint remember_window_position;
extern gint window_pos_x;
extern gint window_pos_y;
extern gint alternate_skin;
extern gint wm_decorations;

	/* miscellaneous */
extern gchar *current_dir;
extern gchar *current_file;
extern gchar *current_path;
extern gchar *mixer_command;
extern gint mixer_device_id;

extern gint status;
extern gint repeat_mode;
extern gint shuffle_mode;
extern gint save_mode_in_playlist;
extern gint obey_mode_in_playlist;

extern gchar *preset_name[];
extern gchar *preset_file[];
extern gint play_presets;
extern gint show_remaining_time;

extern int pid;

extern gint new_song;

extern gchar *default_skin;

extern gint pos_slider_in_drag;

extern gint show_extra_winamp_info;
extern gint double_size_mode;

extern gint debug_mode;
extern gint debug_skin;

extern gchar **gqmpeg_icon_xpm;

extern gint allow_ipc;

	/* main.c routines */
int isplaylist(char *fn);
gchar *filename_from_path(char *t);
gint dot_file_check(gchar *file);
gint filter_file_check(gchar *file);
void rebuild_filter();
void set_song_info(gint n);
gchar *get_song_title(gint n);
void play_button_pressed();
void stop_button_pressed();
void pause_button_pressed();
void prev_button_dn();
void prev_button_up();
void next_button_dn();
void next_button_up();
void next_button_pressed();
void prev_button_pressed();
void rewind_button_pressed();
void fastforward_button_pressed();
void shuffle_button_pressed();
void repeat_button_pressed();
void rew_button_dn();
void rew_button_up();
void ff_button_dn();
void ff_button_up();
void iconify_button_pressed();
void mixer_button_pressed();
void current_song_info_pressed();
void set_scroll_right();
void set_scroll_left();
void position_slider_moved(float pos);
void position_slider_released(float pos);
void volume_slider_moved(float pos);
void volume_button_up_pressed();
void volume_button_dn_pressed();
void volume_button_released();
void balance_slider_moved(float pos);
void balance_button_l_pressed();
void balance_button_r_pressed();
void balance_button_released();
void time_button_pressed();
void preset_button_1_pressed();
void preset_button_2_pressed();
void preset_button_3_pressed();
void preset_button_4_pressed();
void preset_button_5_pressed();
void preset_button_6_pressed();
void preset_button_7_pressed();
void preset_button_8_pressed();
void preset_button_9_pressed();
void preset_button_10_pressed();
void dnd_set_drop(GtkWidget *widget, GdkWindow *window);
void destroy();
int main (int argc, char *argv[]);

	/* skinui.c routines */
GdkPixmap *get_pixmap_from_data(gchar **data);
GdkPixmap *get_pixmap_from_file(gchar *path);
GdkBitmap *get_bitmap_from_file(gchar *path);
void redraw_skin();
ButtonData *new_button(GdkPixmap *pixmap, gint prelight, gint light, gint x, gint y,
		void (*click_func)(void *), void (*press_func)(void *), void (*release_func)(void *));
ButtonData *new_button_from_data(gchar **data, gint prelight, gint light, gint x, gint y,
		void (*click_func)(void *), void (*press_func)(void *), void (*release_func)(void *));
ButtonData *new_button_from_file(gchar *file, gint prelight, gint light, gint x, gint y,
		void (*click_func)(void *), void (*press_func)(void *), void (*release_func)(void *));
DigitData *new_digit(GdkPixmap *pixmap);
DigitData *new_digit_from_data(gchar **data);
DigitData *new_digit_from_file(gchar *file);
FontData * new_font(GdkPixmap *pixmap, gint length, gint x, gint y);
FontData * new_font_from_data(gchar **data, gint length, gint x, gint y);
FontData * new_font_from_file(gchar *file, gint length, gint x, gint y);
NumberData *new_number(DigitData *digits, gint length, gint zeros, gint x, gint y);
NumberData *new_number_from_data(gchar **data, gint length, gint zeros, gint x, gint y);
NumberData *new_number_from_file(gchar *file, gint length, gint zeros, gint x, gint y);
ItemData *new_item(GdkPixmap *pixmap, gint sections, gint x, gint y);
ItemData *new_item_from_data(gchar **data, gint sections, gint x, gint y);
ItemData *new_item_from_file(gchar *file, gint sections, gint x, gint y);
SliderData *new_slider(GdkPixmap *pixmap, gint prelight, gint verticle, gint reversed,
			gint length, gint x, gint y, ItemData *item,
			void (*press_func)(float), void (*release_func)(float),
			void (*move_func)(float));
SliderData *new_slider_from_data(gchar **data, gint prelight, gint verticle, gint reversed,
			gint length, gint x, gint y, ItemData *item,
			void (*press_func)(float), void (*release_func)(float),
			void (*move_func)(float));
SliderData *new_slider_from_file(gchar *file, gint prelight, gint verticle, gint reversed,
			gint length, gint x, gint y, ItemData *item,
			void (*press_func)(float), void (*release_func)(float),
			void (*move_func)(float));
SkinData *new_skin();
void sync_window_to_skin();
void free_skin(SkinData *s);
void draw_number(NumberData *number, gint n);
void draw_font(FontData *font, gchar *text);
void draw_item(ItemData *item, gint section);
void change_button_light(ButtonData *button, gint lit);
void draw_button(ButtonData *button, gint prelight, gint pressed, gint force);
void draw_slider(SliderData *slider, float position, gint absolute, gint move,
		gint prelight, gint pushed, gint force);
void setup_display();
void mode_button_pressed();

	/* skinutil.c routines */
gint change_to_skin(gchar *file);

	/* skindef.c routines */
SkinData *load_default_normal_skin();
SkinData *load_default_small_skin();

	/* filelist.c routines */
void filelist_init();
void filelist_clear();
void filelist_add_item(gchar *fn);
void filelist_insert_item(gchar *fn, gint n);
void filelist_remove_item(gint n);
void filelist_swap_items(gint n1, gint n2);
void filelist_sort();
gchar *filelist_get_item(gint n);
void dirlist_init();
void dirlist_clear();
void dirlist_add_item(gchar *fn);
void dirlist_insert_item(gchar *fn, gint n);
void dirlist_remove_item(gint n);
void dirlist_swap_items(gint n1, gint n2);
void dirlist_sort();
gchar *dirlist_get_item(gint n);

	/* playlist.c routines */
void shuffle_list_create(gint start_with_current);
void shuffle_list_destroy();
gint shuffle_list_get_first();
gint shuffle_list_get_next(gint p);
gint shuffle_list_get_prev(gint p);
void update_list_highlight(GtkWidget *clistw, gint old, gint new, gint move);
gint playlist_load(gchar *fn, gint append);
gint playlist_save(gchar *fn);
void playlist_init();
void playlist_clear();
void playlist_add_item(gchar *fn);
void playlist_insert_item(gchar *fn, gint n);
void playlist_remove_item(gint n);
void playlist_swap_items(gint n1, gint n2);
gchar *playlist_get_item(gint n);
gchar *playlist_get_title(gint n);
Song_Data *playlist_get_data(gint n);
gint playlist_get_index(gchar *fn);
void playlist_update_tags();
void playlist_update_headers();
void playlist_add_song(gchar *fn);
void playlist_insert_song(gchar *fn, gint n);
void playlist_remove_song(gchar *fn, gint n);
void playlist_replace_song(gchar *old_fn, gchar *new_fn);
void set_song(int n);
void add_dir_pressed();
void add_dir_recursive_pressed();

#ifdef __GTK_DND_H__

#else
void dnd_data_dropped(GtkWidget *widget, GdkEventDropDataAvailable *event);
#endif

gint load_playlist_from_file(gchar *fn);
gint append_playlist_from_file(gchar *fn);
void load_pressed();
void append_pressed();
void load_button_pressed();
void add_button_pressed();
void remove_button_pressed();
void insert_button_pressed();
void add_play_button_pressed();
void pos_play_button_pressed();
void playlist_songinfo_pressed();
void filelist_songinfo_pressed();
void playlist_move_pressed();
void filelist_move_pressed();
void playlist_copy_pressed();
void filelist_copy_pressed();
void playlist_rename_pressed();
void filelist_rename_pressed();
void playlist_delete_pressed();
void filelist_delete_pressed();
void refresh_dir();
void show_playlist_length();
void show_playlist();

	/* playerio.c routines */
void start_playing_file();
void stop_playing_file();

	/* display.c routines */
void update_slider_display();
void update_volume_display();
void update_balance_display();
void update_shuffle_display();
void update_repeat_display();
void update_status_display();
void update_frame_count_display(gint n);
void draw_small_digits(gint n,gint l,gint z,gint x,gint y);
void update_cpu_display(gfloat percentf);
void update_time_count(gint count, gint remain);
void update_song_number(gint n);
void update_song_count(gint n);
void update_io_information();
void redraw_display();
void redraw_name(gchar *t);
void update_all_display_info();

	/* fileops.c routines */
gchar *homedir();
int isfile(char *s);
int isdir(char *s);
int filesize(char *s);
void get_current_dir();
void set_current_dir(char *s);
void read_dir();
int copy_file(char *s, char *t);
int move_file(char *s, char *t);


	/* config.c routines */
void show_about_info();
void show_configuration_window();

	/* gqmpegrc.c routines */
void save_options();
void load_options();

	/* globals.c routines */
void init_vars();

	/* tagutil.c routines */
gchar* get_id3_genre(gint g);
ID3_Data *get_id3_tag(gchar *path);
gint set_id3_tag(gchar *path, gchar *title, gchar *artist, gchar *album,
		 gchar *year, gchar *comment, guint8 genre);
void free_id3_tag(ID3_Data *d);

	/* hdrutil.c routines */
Mpg_Data *get_mpg_header_info(gchar *path);

	/* menu.c routines */
void create_preset_menu();
GtkWidget *create_main_menu();
GtkWidget *create_dir_menu();
GtkWidget *create_file_menu();
GtkWidget *create_file_menu_list();
GtkWidget *create_playlist_menu();

	/* songinfo.c routines */
void view_song_info(gchar *file);

	/* skinwamp.c routines */
SkinData *load_wamp_normal_skin(gchar *path);
SkinData *load_wamp_small_skin(gchar *path);

	/* mixer.c routines */
void mixer_init(gint init_device_id);
void set_volume(gint vol);
gint get_volume();
void set_balance(gint bal);
gint get_balance();
GList *get_mixer_device_list();
gint get_mixer_device_id(gchar *device_name);
void set_mixer_device(gint device_id);

	/* utildlg.c routines */
void warning_dialog(gchar *title, gchar *message);
void file_rename_dialog(gchar *fn);
void file_delete_dialog(gchar *fn);
void file_move_dialog(gchar *fn);
void file_copy_dialog(gchar *fn);
void playlist_save_as_dialog(gchar *fn);
void playlist_load_dialog(gchar *fn);
void playlist_append_dialog(gchar *fn);

	/* movedlg.c routines */
GtkWidget *destination_widget_new_with_files(gchar *path, gchar *filter, GtkWidget *entry);
GtkWidget *destination_widget_new(gchar *path, GtkWidget *entry);

	/* tabcomp.c routines */
void tab_completion_add_to_entry(GtkWidget *entry, void (*enter_func)(gchar *, gpointer), gpointer data);
gchar *remove_trailing_slash(gchar *path);

	/* ipc.c routines */
void ipc_send();
void ipc_on();
void ipc_off();

/* end of header */
