/*###################################################################*/
/*##                         GQmpeg 0.4.5                          ##*/
/*##      This software is Copyright (C) 1999 by John Ellis.       ##*/
/*## This software falls under the GNU Public License. Please read ##*/
/*##              the COPYING file for more information            ##*/
/*###################################################################*/

#include "gqmpeg.h"

/* note:
 * this is really cheesy right now, basically to send a command to a running
 * gqmpeg, create a file in ~/.gqmpeg/command and place in it a command
 * string, the best way would be:
 *   echo "play" > ~/.gqmpeg/commands
 * the file is deleted when gqmpeg parses/executes the command.
 * existance of the file is checked every 3 seconds.
 */

static gint ipc_timeout_id = -1;
static gchar *ipc_file = NULL;

void ipc_send(gchar *command)
{
	FILE *f;

	if (ipc_file)
		{
		f = fopen(ipc_file, "w");
		}
	else
		{
		gchar *path = g_strconcat(homedir(), "/.gqmpeg/command", NULL);
		f = fopen(path, "w");
		g_free(path);
		}

	if (!f)
		{
		printf("error opening command file %s\n", ipc_file);
		return;
		}

	fprintf(f, command);

	fclose(f);
}

static void ipc_do()
{
	FILE *f;
	gchar i_buf[1025];
	i_buf[1024] = '\0'; /* paranoid safety */

	f = fopen(ipc_file, "r");
	if (!f)
		{
		printf("error opening command file %s\n", ipc_file);
		return;
		}

	while (fgets(i_buf,1024,f))
		{
		if (!strncmp(i_buf, "play", 4))
			{
			play_button_pressed();
			}
		else if (!strncmp(i_buf, "stop", 4))
			{
			stop_button_pressed();
			}
		else if (!strncmp(i_buf, "pause", 5))
			{
			pause_button_pressed();
			}
		else if (!strncmp(i_buf, "next", 4))
			{
			next_button_dn();
			next_button_pressed();
			}
		else if (!strncmp(i_buf, "prev", 4))
			{
			prev_button_dn();
			prev_button_pressed();
			}
		else if (!strncmp(i_buf, "shoutcast", 9) && strlen(i_buf) > 11)
			{
			gchar *host = i_buf + 11;
			stop_button_pressed();
			playlist_song = -1;
/*			g_free(song_filename);*/
			song_filename = g_strconcat("shoutcast ", host, NULL);
/*			g_free(song_path);*/
			song_path = g_strdup(host);
			start_playing_file();
			}
		else if (!strncmp(i_buf, "help", 4))
			{
			printf("available ipc commands:\n");
			printf("play, pause, stop, next, prev\n");
			printf("shoutcast http://server:port\n");
			}
		else
			{
			printf("unknown ipc command: `%s'\n", i_buf);
			}
		}

	fclose(f);

	if ((unlink(ipc_file) < 0))
		{
		printf("unable to remove command file %s\n", ipc_file);
		}
}

static gint ipc_check()
{
	if (!ipc_file) return FALSE;
	if (isfile(ipc_file)) ipc_do();
	return TRUE;
}

void ipc_on()
{
	if (ipc_timeout_id > -1) return;
	ipc_timeout_id = gtk_timeout_add(3000, ipc_check, NULL);
	ipc_file = g_strconcat(homedir(), "/.gqmpeg/command", NULL);
}

void ipc_off()
{
	if (ipc_timeout_id > -1)
		{
		gtk_timeout_remove(ipc_timeout_id);
		ipc_timeout_id = -1;
		}
	g_free(ipc_file);
	ipc_file = NULL;
}

