/*###################################################################*/
/*##                         GQmpeg 0.4.5                          ##*/
/*##    This software is Copyright (C) 1998, 1999 by John Ellis.   ##*/
/*## This software falls under the GNU Public License. Please read ##*/
/*##              the COPYING file for more information            ##*/
/*###################################################################*/

#include "gqmpeg.h"

/* these two includes are _only_ used in iconify_button_pressed() */
#include <gdk/gdkx.h>
#include <X11/Xlib.h>

static gint fast_advance = FALSE;
static gint advance_max_frame;
static gint advance_max_sec;
static gint advance_pos;
static gint song_advance = FALSE;
static gint song_advance_pos;
static gint name_scroll = -1;
static gint name_scroll_pos = 0;
static gint update_timeout_id;
static gint vol_btn_down = 0;
static gint bal_btn_down = 0;

static void advance_button_released();
static gfloat check_child_cpu_usage(int childs_pid);
static gint update_condition();
static gint advancing_callback();
static void advance_start();
static void adjust_volume_by(gint n);
static void adjust_balance_by(gint n);
static void load_preset(gint n);

int isplaylist(char *fn)
{
	int l = strlen(fn);

	if (l < 7) return FALSE;
	if (!strncasecmp(fn + l - 7,".gqmpeg",7)) return TRUE;
	return FALSE;
}

gchar *filename_from_path(char *t)
{
	char *p;

	p = t + strlen(t);
	while(p > t && p[0] != '/') p--;
	if (p[0] == '/') p++;
	return p;
}

gint dot_file_check(gchar *file)
{
	if (!strcmp(file,".") || !strcmp(file,"..")) return 1;
	if (file[0] == '.') return 0;
	return 1;
}

gint filter_file_check(gchar *file)
{
	gchar *f_ptr = filter;
	gchar *file_ptr;
	gchar *strt_ptr;
	int i;

	while (f_ptr < filter + strlen(filter))
		{
		strt_ptr = f_ptr;
		i=0;
		while (*f_ptr != ';' && *f_ptr != '\0')
			{
			f_ptr++;
			i++;
			}
		f_ptr++;
		file_ptr = file + strlen(file) - i;
		if (!strncasecmp(file_ptr, strt_ptr, i)) return 1;
		}
	return 0;
}

void rebuild_filter()
{
	strcpy(filter, "");
	if (FILTER_INCLUDE_MP3)
		strcat (filter, ".mp2;.mp3;.mpg;.mpeg;");
	if (FILTER_INCLUDE_GQMPEG)
		strcat (filter, ".gqmpeg;");
}

/* the ifdef's are getting a little heavy here, be careful */

#ifndef HZ
#define HZ 100 /* for FreeBSD? */
#endif

static gfloat check_child_cpu_usage(int childs_pid)
{
/* needed for SGI IRIX 5.3 and 6.5 */
#ifdef SGI
	int fd;
	int retval;
	prusage_t p;
#endif

/* needed to get cpu % on Sun Solaris */
#ifdef sun
	gfloat percent_of_cpu = 0.0;
	prpsinfo_t psinfo;
	char procname[20];
	int  procfile;
  
	sprintf(procname, "/proc/%.5d", childs_pid);

	if ( (procfile = open(procname, O_RDONLY)) == -1 )
		percent_of_cpu = 0.0;
	else
		{
		if ( ioctl(procfile, PIOCPSINFO, &psinfo) == -1 )
			percent_of_cpu = 0.0;
		else
			percent_of_cpu = ( ((gfloat) psinfo.pr_pctcpu) *100.0 / 32768.0 ); 
		close(procfile);
		}
#else
	struct timeval time;
	static struct timeval oldtime;
	struct timezone timez;
	static gfloat old_child_time;

	gchar s[32], str[256];
	gchar *strp;
	glong child_time;
	gfloat child, cpu;
	gfloat percent_of_cpu;
	int i;

	FILE *f;

	if (childs_pid <=0) return 0;

	gettimeofday(&time, &timez);

	sprintf(s,"/proc/%d/stat",childs_pid);

/* needed for SGI IRIX 5.3 and 6.5 */
#ifdef SGI
	sprintf(s,"/proc/pinfo/%05d",childs_pid);

	fd = open(s, O_RDONLY);
	if (fd < 0) return 0;
	retval = ioctl(fd, PIOCUSAGE, &p);
	if (retval < 0)
		{
		perror(s);
		close(fd);
		return 0;
		}
	close(fd);
	child_time = (p.pu_utime.tv_sec * 100) + p.pu_utime.tv_nsec / 10000000;
#else
	f = fopen(s,"r");

	/* open failed */
	if (!f) return 0;

	fread(str,1,255,f);
	fclose(f);

	strp = str;
	for (i=0; i < 13; i++)
		{
		while ( strp[0] != ' ' && strp < &str[0] + strlen(str)) strp ++;
		strp++;
		}
	child_time = strtol(strp,NULL,0);
#endif

	child = child_time - old_child_time;
	cpu = (time.tv_sec - oldtime.tv_sec)
		+ (float) (time.tv_usec - oldtime.tv_usec) / 1000000.0;

	oldtime.tv_sec = time.tv_sec;
	oldtime.tv_usec = time.tv_usec;
	old_child_time = child_time;

	percent_of_cpu = (child / HZ) / cpu * 100;
#endif

	if (percent_of_cpu > 99)
	    percent_of_cpu = 99;

	return percent_of_cpu;
}

void set_song_info(gint n)
{
	Song_Data *d = NULL;
	if (n >= playlist_count && playlist_count > 0) return;

	if (n < 0 || playlist_count < 1)
		{
		song_path = "";
		song_filename = "playlist empty";
		}
	else
		{
		d = playlist_get_data(n);
		song_path = playlist_get_item(n);
		song_filename = filename_from_path(song_path);
		}

	/* hope this works: */
	frames = 0;
	frames_remaining = 0;
	seconds = 0;
	seconds_remaining = 0;
	mpeg_version = 0;
	mpeg_layer = 0;
	mpeg_bitrate = 0;
	mpeg_mode = 0;
	mpeg_channels = 0;
	mpeg_hz = 0;
	output_channels = 0;
	output_hz = 0;
	output_bits = 0;
	output_conversion = 0;

	id3_title = NULL;
	g_free(id3_artist);
	id3_artist = NULL;
	g_free(id3_album);
	id3_album = NULL;
	g_free(id3_year);
	id3_year = NULL;
	g_free(id3_comment);
	id3_comment = NULL;
	id3_genre = -1;
}

gchar *get_song_title(gint n)
{
	if (n >= playlist_count && playlist_count > 0) return "";

	if (n < 0) return song_filename;

	return playlist_get_title(n);
}

static gint update_condition()
{
	static gint old_song_advance;
	static gint status_chk = -1;
	static gint update_counter = 0;
	float cpu_percent;

	if (fast_advance != 0) return TRUE;

	if (song_advance != 0)
		{
		if (song_advance < 8 && song_advance > -8)
			{
			if (song_advance > 0)
				song_advance++;
			else
				song_advance--;
			}
		else
			{
			static int trip_cnt;
			if (song_advance < 32 && song_advance > -32)
				{
				trip_cnt++;
				if (trip_cnt > 2) trip_cnt = 0;
				if (song_advance > 0)
					song_advance++;
				else
					song_advance--;
				}
			else trip_cnt = 0;
			if (trip_cnt == 0)
				{
				if (shuffle_mode)
					{
					if (song_advance > 0)
						song_advance_pos = shuffle_list_get_next(song_advance_pos);
					else
						song_advance_pos = shuffle_list_get_prev(song_advance_pos);
					}
				else
					{
					if (song_advance > 0 && song_advance_pos < playlist_count - 1)
						song_advance_pos++;
					if (song_advance < 0 && song_advance_pos > 0)
						song_advance_pos--;
					}
				update_song_number(song_advance_pos);
				redraw_name(get_song_title(song_advance_pos));
				redraw_display();
				}
			}
		old_song_advance = song_advance;
		}
	else if (old_song_advance != song_advance)
		{
		old_song_advance = song_advance;
		if (song_advance_pos != playlist_song)
			{
			update_song_number(playlist_song);
			redraw_name(get_song_title(playlist_song));
			redraw_display();
			}
		}

	if (vol_btn_down !=0)
		{
		if (vol_btn_down == 1)
			adjust_volume_by(3);
		else
			adjust_volume_by(-3);
		}

	if (bal_btn_down !=0)
		{
		if (bal_btn_down == 1)
			adjust_balance_by(-3);
		else
			adjust_balance_by(3);
		}

	/* this is true once the song's header is parsed from mpg123's output */
	if (new_song)
		{
		update_io_information();
		redraw_name(get_song_title(playlist_song));
		new_song = FALSE;
		}

	/* title */
	if (name_scroll > 0 && skin->title)
		{
		FontData *font = skin->title;
		gchar *title = get_song_title(playlist_song);
		if (name_scroll_pos < strlen(title) - font->length && strlen(title) > font->length)
			{
			name_scroll_pos++;
			redraw_name(title + name_scroll_pos);
			}
		}
	if (name_scroll < 0 && name_scroll_pos >0 && skin->title)
		{
		FontData *font = skin->title;
		gchar *title = get_song_title(playlist_song);
		name_scroll_pos--;
		if (name_scroll_pos < strlen(title) - font->length)
			redraw_name(title + name_scroll_pos);
		else
			{
			name_scroll_pos = 0;
			redraw_name(title);
			}
		}

	update_counter++;

	if (status == STATUS_PLAY)
		{
		static int old_seconds;

		if (debug_mode) putchar('u');

		/* display */
		if (seconds != old_seconds && !pos_slider_in_drag)
			{
			if (update_counter > 15)
				{
				update_counter = 0;
				cpu_percent = check_child_cpu_usage(pid);
				update_cpu_display(cpu_percent);
				}
			update_time_count(seconds, seconds_remaining);
			update_frame_count_display(frames);
			update_slider_display();
			redraw_display();
			}
		old_seconds = seconds;
		}
	

	if (status == status_chk)
		{
		if (debug_mode) putchar('c');
		return TRUE;
		}

	if (status == STATUS_NEXT)
		{
		gint new_pos = -1;
		/* if repeat is on, circle to beginning */
		if (repeat_mode && playlist_song >= 0 && playlist_count - 1 == playlist_song)
			new_pos = 0;
		/* simply increment to the next song */
		if (playlist_song >= 0 && playlist_count - 1 > playlist_song)
			new_pos = playlist_song + 1;
		/* select next random song if shuffle is on */
		if (shuffle_mode)
			{
			gint p = shuffle_list_get_next(playlist_song);
			if ( p == playlist_song)
				if (repeat_mode)
					{
					shuffle_list_create(FALSE);
					new_pos = shuffle_list_get_first();
					}
				else
					new_pos = -1;
			else
				new_pos = p;
			}
		if (new_pos >= 0)
			{
			playlist_song = new_pos;
			set_song_info(playlist_song);
			start_playing_file();
			update_song_number(playlist_song);
			redraw_display(get_song_title(playlist_song));
			}
		else
			status = STATUS_STOP;

		}
	if (status == STATUS_PLAY)
		{
		if (playlist_song >= 0)
			{
			update_song_number(playlist_song);
			}

		update_status_display();

		redraw_name(get_song_title(playlist_song));
		redraw_display();
	}
	if (status == STATUS_STOP)
		{
		update_status_display();

		update_cpu_display(0);
		update_time_count(0, 0);
		update_frame_count_display(0);
		update_slider_display();
		redraw_display();
		if (playlist_count <= 0)
			{
			playlist_song = -1;
			set_song_info(playlist_song);
			redraw_name(get_song_title(playlist_song));
			}
		}
	if (status == STATUS_PAUSE)
		{
		update_status_display();

		update_cpu_display(0);
		redraw_display();
		}
	status_chk = status;
	return 1;
}

void play_button_pressed()
{
	if (playlist_count < 1) return;
	if (playlist_song > playlist_count - 1 || playlist_song < 0) playlist_song = 0;
	if (status == STATUS_PLAY)
		{
		stop_playing_file();
		status = STATUS_PAUSE;
		return;
		}
	if (status == STATUS_PAUSE)
		{
		start_playing_file();
		return;
		}
	set_song_info(playlist_song);
	frames = 0;
	start_playing_file();
}

void stop_button_pressed()
{
	if (status == STATUS_STOP) return;
	if (status == STATUS_PLAY)
		{
		stop_playing_file();
		frames = 0;
		}
	status = STATUS_STOP;
}

void pause_button_pressed()
{
	if (status == STATUS_STOP) return;
	if (status == STATUS_PLAY)
		{
		stop_playing_file();
		status = STATUS_PAUSE;
		}
	else
		{
		start_playing_file();
		}
}

void next_button_pressed()
{
	if (playlist_count > 0 && playlist_song != song_advance_pos)
		{
		playlist_song = song_advance_pos;
		song_advance = 0;
		if (status == STATUS_PAUSE)
			{
			status = STATUS_STOP;
			}
		if (status == STATUS_STOP)
			{
			set_song_info(playlist_song);
			update_song_number(playlist_song);
			redraw_name(get_song_title(playlist_song));
			redraw_display();
			return;
			}
		if (status == STATUS_PLAY)
			{
			stop_playing_file();
			status = STATUS_STOP;
			frames = 0;
			set_song_info(playlist_song);
			start_playing_file();
			redraw_name(get_song_title(playlist_song));
			update_song_number(playlist_song);
			redraw_display();
			}
		}
}

void prev_button_pressed()
{
	next_button_pressed();
}

static gint advancing_callback()
{
	gint tsec;
	if (fast_advance == 0) return FALSE;

	/* fast forwarding */
	if (fast_advance > 0)
		{
		if (fast_advance < 35)
			{
			advance_pos += 15;
			fast_advance++;
			}
		else
			{
			advance_pos += 50;
			}
		if (advance_pos > advance_max_frame - 200) advance_pos = 0;
		}
	/* rewinding */
	if (fast_advance < 0)
		{
		if (fast_advance > -35)
			{
			advance_pos -= 15;
			fast_advance--;
			}
		else
			{
			advance_pos -= 50;
			}
		if (advance_pos < 0) advance_pos = advance_max_frame - 200;
		}
	tsec = (float)advance_pos / advance_max_frame * advance_max_sec;
	update_time_count(tsec, advance_max_sec - tsec);
	update_frame_count_display(advance_pos);
	draw_slider(skin->position, (float)advance_pos / advance_max_frame,
		FALSE, TRUE, FALSE, FALSE, FALSE);
	redraw_display();
	return TRUE;
}

static void advance_start()
{
	advance_max_frame = frames + frames_remaining;
	advance_max_sec = seconds + seconds_remaining;
	advance_pos = frames;
	gtk_timeout_add(100, (GtkFunction)advancing_callback, NULL);
}

void rewind_button_pressed()
{
	if (fast_advance != FALSE) return;
	if (status != STATUS_PLAY && status != STATUS_PAUSE) return;
	fast_advance = -1;
	advance_start();
}

void fastforward_button_pressed()
{
	if (fast_advance != FALSE) return;
	if (status != STATUS_PLAY && status != STATUS_PAUSE) return;
	fast_advance = 1;
	advance_start();
}

static void advance_button_released()
{
	if (fast_advance == FALSE) return;
	fast_advance = FALSE;
	if (status != STATUS_PLAY && status != STATUS_PAUSE) return;
	if (status == STATUS_PLAY)
		{
		stop_playing_file();
		status = STATUS_STOP;
		}
	frames = advance_pos;
	frames_remaining = advance_max_frame - frames;
	seconds = (float)advance_pos / advance_max_frame * advance_max_sec;
	seconds_remaining = advance_max_sec - seconds;
	if (status != STATUS_PAUSE) start_playing_file();
}

void shuffle_button_pressed()
{
	if (shuffle_mode)
		{
		shuffle_mode = FALSE;
		shuffle_list_destroy();
		}
	else
		{
		shuffle_mode = TRUE;
		shuffle_list_create(TRUE);
		}
	update_shuffle_display();
	redraw_display();
}

void repeat_button_pressed()
{
	if (repeat_mode)
		repeat_mode = FALSE;
	else
		repeat_mode = TRUE;
	update_repeat_display();
	redraw_display();
}

void set_scroll_right()
{
if (debug_mode) printf("in\n");
	name_scroll = 1;
}

void set_scroll_left()
{
if (debug_mode) printf("out\n");
	name_scroll = -1;
}

void prev_button_dn()
{
	if (playlist_count>0)
		song_advance = -1;
	else
		song_advance = 0;
	song_advance_pos = playlist_song;
	if (((song_advance_pos>0) || shuffle_mode) && (playlist_count>0))
		{
		if (shuffle_mode)
			song_advance_pos=shuffle_list_get_prev(song_advance_pos);
		else
			song_advance_pos--;
		update_song_number(song_advance_pos);
		redraw_name(get_song_title(song_advance_pos));
		redraw_display();
		}
}

void prev_button_up()
{
	song_advance = 0;
}

void next_button_dn()
{
	if (playlist_count>0)
		song_advance = 1;
	else
		song_advance = 0;
	song_advance_pos = playlist_song;
	if (((song_advance_pos<playlist_count-1) || shuffle_mode) && (playlist_count>0))
		{
		if (shuffle_mode)
			{
			song_advance_pos=shuffle_list_get_next(song_advance_pos);
			}
		else
			song_advance_pos++;
		update_song_number(song_advance_pos);
		redraw_name(get_song_title(song_advance_pos));
		redraw_display();
		}
}

void next_button_up()
{
	song_advance = 0;
}

void rew_button_dn()
{
	rewind_button_pressed();
}

void rew_button_up()
{
	advance_button_released();
}

void ff_button_dn()
{
	fastforward_button_pressed();
}

void ff_button_up()
{
	advance_button_released();
}

void iconify_button_pressed()
{
	Window xwindow;

	if (!mainwindow->window) return;

	xwindow = GDK_WINDOW_XWINDOW(mainwindow->window);
	XIconifyWindow (GDK_DISPLAY (), xwindow, DefaultScreen (GDK_DISPLAY ()));
}

void mixer_button_pressed()
{
	if (mixer_command)
		{
		gchar *command = g_strconcat(mixer_command, " &", NULL);
		system(command);
		g_free(command);
		}
}

void current_song_info_pressed()
{
	if (playlist_song < 0 || playlist_song > playlist_count) return;
	view_song_info(playlist_get_item(playlist_song));
}

void position_slider_moved(float pos)
{
	gint secs;
	gint frms;
	if (seconds_remaining + seconds == 0 || status == STATUS_STOP) return;
	secs = (float)(seconds_remaining + seconds) * pos;
	frms = (float)(frames_remaining + frames) * pos;
	update_frame_count_display(frms);
	update_time_count(secs, seconds_remaining + seconds - secs);
	redraw_display();
}

void position_slider_released(float pos)
{
	gint secs;
	gint frms;
	gint secs_rm;
	gint frms_rm;
	if (frames_remaining + frames == 0 || status == STATUS_STOP) return;
	secs = (float)(seconds_remaining + seconds) * pos;
	secs_rm = seconds_remaining + seconds - secs;
	frms = (float)(frames_remaining + frames) * pos;
	frms_rm = frames_remaining + frames - frms;
	if (frms > frames_remaining + frames - 15)
		frms = frames_remaining + frames - 15;
	update_frame_count_display(frms);
	update_time_count(secs, secs_rm);
	redraw_display();

	if (status == STATUS_PLAY)
		{
		stop_playing_file();
		status = STATUS_PAUSE;
		frames = frms;
		frames_remaining = frms_rm;
		seconds = secs;
		seconds_remaining = secs_rm;
		start_playing_file();
		status = STATUS_PLAY;
		}
	else
		frames = frms;
		frames_remaining = frms_rm;
		seconds = secs;
		seconds_remaining = secs_rm;
}

void volume_slider_moved(float pos)
{
	set_volume(pos * 100);
}

void volume_button_up_pressed()
{
	vol_btn_down = 1;
	adjust_volume_by(3);
}

void volume_button_dn_pressed()
{
	vol_btn_down = 2;
	adjust_volume_by(-3);
}

void volume_button_released()
{
	vol_btn_down = 0;
}

static void adjust_volume_by(gint n)
{
	gint vol = get_volume();
	vol += n;
	if (vol < 0) vol = 0;
	if (vol > 100) vol = 100;
	set_volume(vol);
	update_volume_display();
	redraw_display();
}

void balance_slider_moved(float pos)
{
	gint p = pos * 100;
	if (p > 44 && p < 56) p = 50; /* to help balance 'lock' on center */
	set_balance(p);
}

void balance_button_l_pressed()
{
	bal_btn_down = 1;
	adjust_balance_by(-3);
}

void balance_button_r_pressed()
{
	bal_btn_down = 2;
	adjust_balance_by(3);
}

void balance_button_released()
{
	bal_btn_down = 0;
}

static void adjust_balance_by(gint n)
{
	gint bal = get_balance();
	bal += n;
	if (bal < 0) bal = 0;
	if (bal > 100) bal = 100;
	if (bal > 50 - abs(n) && bal < 50 + abs(n)) bal = 50; /* basic balance auto-centering */
	set_balance(bal);
	update_balance_display();
	redraw_display();
}

void time_button_pressed()
{
	show_remaining_time = !show_remaining_time;
	if (status == STATUS_PLAY || status == STATUS_PAUSE)
		{
		update_time_count(seconds, seconds_remaining);
		}
}

/* messy preset handling, but for now... */

static void load_preset(gint n)
{
	if (!preset_file[n]) return;
        if (load_playlist_from_file(preset_file[n]) == TRUE)
                {
                if (play_presets) start_playing_file();
                }
}

void preset_button_1_pressed()
{
	load_preset(0);
}

void preset_button_2_pressed()
{
	load_preset(1);
}

void preset_button_3_pressed()
{
	load_preset(2);
}

void preset_button_4_pressed()
{
	load_preset(3);
}

void preset_button_5_pressed()
{
	load_preset(4);
}

void preset_button_6_pressed()
{
	load_preset(5);
}

void preset_button_7_pressed()
{
	load_preset(6);
}

void preset_button_8_pressed()
{
	load_preset(7);
}

void preset_button_9_pressed()
{
	load_preset(8);
}

void preset_button_10_pressed()
{
	load_preset(9);
}

static void key_pressed(GtkWidget *widget, GdkEventKey *event)
{
	static gint last_key = 0;
	static gint last_state = 0;
	static guint32 last_time = 0;

	/* cheap hacks/checks to avoid problems caused by holding down
	 * keys with fast repeat rates */
	if (event->keyval == last_key && event->state == last_state &&
			last_time + 500 >= event->time)
		{
		return;
		}
	else
		{
		last_key = event->keyval;
		last_state = event->state;
		last_time = event->time;
		}

	if (debug_mode) printf("key pressed: %d\n", event->keyval);

	if (event->state & GDK_CONTROL_MASK)
		{
		switch (event->keyval)
			{
			case 's':
				shuffle_button_pressed();
				break;
			case 'r':
				repeat_button_pressed();
				break;
			case 't':
				time_button_pressed();
				break;
			case 'q':
				destroy();
				break;
			case 'a':
				show_about_info();
				break;
			case 'o':
				show_configuration_window();
				break;
			case 'i':
				iconify_button_pressed();
				break;
			case '-':
				balance_button_l_pressed();
				balance_button_released();
				break;
			case '+': case '=':
				balance_button_r_pressed();
				balance_button_released();
				break;
			default:
				break;
			}
		}
	else
		{
		switch (event->keyval)
			{
			case 'p': case ' ':
				play_button_pressed();
				break;
			case 's':
				stop_button_pressed();
				break;
			case 'n':
				next_button_dn();
				next_button_pressed();
				break;
			case 'b':
				prev_button_dn();
				prev_button_pressed();
				break;
			case 'l':
				show_playlist();
				break;
			case 'm':
				mixer_button_pressed();
				break;
			case 'w':
				mode_button_pressed();
				break;
			case 'i':
				current_song_info_pressed();
				break;
			case '-':
				volume_button_dn_pressed();
				volume_button_released();
				break;
			case '+': case '=':
				volume_button_up_pressed();
				volume_button_released();
				break;
			case '1':
				preset_button_1_pressed();
				break;
			case '2':
				preset_button_2_pressed();
				break;
			case '3':
				preset_button_3_pressed();
				break;
			case '4':
				preset_button_4_pressed();
				break;
			case '5':
				preset_button_5_pressed();
				break;
			case '6':
				preset_button_6_pressed();
				break;
			case '7':
				preset_button_7_pressed();
				break;
			case '8':
				preset_button_8_pressed();
				break;
			case '9':
				preset_button_9_pressed();
				break;
			case '0':
				preset_button_10_pressed();
				break;
			default:
				break;
			}
		}
}

void dnd_set_drop(GtkWidget *widget, GdkWindow *window)
{
#ifdef __GTK_DND_H__

#else
        static char *drop_types[]=
        {
        "url:ALL"
        };

        gdk_window_dnd_drop_set(window,TRUE,drop_types,1,FALSE);
        gtk_signal_connect(GTK_OBJECT(widget),"drop_data_available_event",
                GTK_SIGNAL_FUNC (dnd_data_dropped), NULL);
#endif
}

static void set_icon_for_window(GtkWidget *window)
{
	GdkPixmap *icon = NULL;
	GdkBitmap *mask = NULL;
	GtkStyle *style = gtk_widget_get_style(window);
	icon = gdk_pixmap_create_from_xpm_d(window->window, &mask,
		&style->bg[GTK_STATE_NORMAL], (gchar **)gqmpeg_icon_xpm);
	gdk_window_set_icon(window->window, NULL, icon, mask);
}

static gint send_if_ipc_match(gchar *short_str, gchar *long_str, gchar *text, gchar *command)
{
	if (!strcmp(text, short_str) || (long_str && !strcmp(text, long_str)))
		{
		ipc_send(command);
		return TRUE;
		}

	return FALSE;
}

static gint check_for_ipc_command(gchar *text)
{
	if (send_if_ipc_match("-p", "--play", text, "play")) return TRUE;
	if (send_if_ipc_match("-s", "--stop", text, "stop")) return TRUE;
	if (send_if_ipc_match("-n", "--next", text, "next")) return TRUE;
	if (send_if_ipc_match("-b", "--back", text, "prev")) return TRUE;
	if (send_if_ipc_match("-ps", "--pause", text, "pause")) return TRUE;

	return FALSE;
}

void destroy()
{
	gchar *session_file;
	gint x = 0;
	gint y = 0;

	stop_playing_file();

	gdk_window_get_position (mainwindow->window, &x, &y);
	window_pos_x = x;
	window_pos_y = y;

	if (skin_small && skin == skin_small)
		alternate_skin = TRUE;
	else
		alternate_skin = FALSE;

	initial_playlist_position = playlist_song;

	save_options();

	/* save playlist between sessions (even if option not enabled) */
	session_file = g_strconcat(homedir(), "/.gqmpeg/session-playlist.gqmpeg", NULL);
	playlist_save(session_file);
	g_free(session_file);

	gtk_main_quit();
}

int main (int argc, char *argv[])
{
	gchar *sbuf;
	gchar *command_line_dir;
	gchar *command_line_path;
	gchar *skin_file = NULL;
	gchar *default_skin_path = NULL;
	gchar *rcdir;
	gchar *hmdir;
	int l;
	int i;

	init_vars();

	gtk_init (&argc, &argv);
	gdk_imlib_init();

	/* setup random seed for shuffle mode */
	srand (time (0));

	/* check that we can find a valid home directory, if not exit semi-gracefully */
	hmdir = homedir();
	if (!hmdir)
		{
		printf("Unable to determine the home directory\n");
		return 0;
		}
	else if (!isdir(hmdir))
		{
		printf("Could not find home directory: %s\n", hmdir);
		return 0;
		}

	/* create gqmpeg dir */
	rcdir = g_strconcat(homedir(), "/.gqmpeg", NULL);
	if (!isdir(rcdir))
		{
		printf("Creating gqmpeg dir:%s\n", rcdir);
		if ( (mkdir (rcdir, 0755) < 0) )
			printf("Could not create dir:%s\n", rcdir);
		}
	g_free(rcdir);

	/* create gqmpeg skin dir */
	rcdir = g_strconcat(homedir(), "/.gqmpeg/skins", NULL);
	if (!isdir(rcdir))
		{
		printf("Creating gqmpeg skin dir:%s\n", rcdir);
		if ( (mkdir (rcdir, 0755) < 0) )
			printf("Could not create dir:%s\n", rcdir);
		}
	g_free(rcdir);

	/* create gqmpeg playlists dir */
	rcdir = g_strconcat(homedir(), "/.gqmpeg/playlists", NULL);
	if (!isdir(rcdir))
		{
		printf("Creating gqmpeg playlist dir:%s\n", rcdir);
		if ( (mkdir (rcdir, 0755) < 0) )
			printf("Could not create dir:%s\n", rcdir);
		}
	g_free(rcdir);

	rcdir = g_strconcat(homedir(), "/.gqmpeg/gtkrc", NULL);
	gtk_rc_parse (rcdir);
	g_free(rcdir);

	for (i=0; i<10; i++)
		{
		preset_name[i] = NULL;
		preset_file[i] = NULL;
		}

	load_options();

	if (!mixer_command) mixer_command = g_strdup("xmixer");

	playlist_init();

	mixer_init(mixer_device_id);

	if (initial_playlist_mode == PLAYLIST_MODE_EMPTY)
		{
		}
	if (initial_playlist_mode == PLAYLIST_MODE_SESSION)
		{
		gchar *session_file;
		session_file = g_strconcat(homedir(),
			"/.gqmpeg/session-playlist.gqmpeg", NULL);
		if (isfile(session_file))
			{
			if (playlist_load(session_file, FALSE) && start_playing_on_startup)
				{
				if (initial_playlist_position < playlist_count)
					{
					playlist_song = initial_playlist_position;
					set_song_info(playlist_song);
					}
				}
			}
		else
			{
			printf("Could not load session-playlist file\n");
			g_free(session_file);
			session_file = g_strconcat(homedir(),
						"/.playlist.gqmpeg", NULL);
			if (isfile(session_file))
				{
				playlist_load(session_file, FALSE);
				printf("Use playlist-session file from old location\n");
				}
			}
		g_free(session_file);
		if (playlist_pathname) g_free(playlist_pathname);
		playlist_pathname = g_strconcat(homedir(), "/.gqmpeg/playlists/Untitled.gqmpeg", NULL);
		if (playlist_filename) g_free(playlist_filename);
		playlist_filename = g_strdup("Untitled.gqmpeg");
		}
	if (initial_playlist_mode == PLAYLIST_MODE_FILE)
		{
		playlist_load(initial_playlist_pathname, FALSE);
		}

	get_current_dir();
	command_line_dir = g_strdup(current_dir);

	if (initial_directory_enable)
		{
		if (isdir(initial_directory_path))
			{
			set_current_dir(initial_directory_path);
			}
		else
			{
			get_current_dir();
			printf("config file contains invalid startup path:\n%s\n",initial_directory_path);
			}
		}
	else
		{
		get_current_dir();
		}

	if (argc > 1)
		{
		i = 1;
		while (i < argc)
			{
			command_line_path = g_strconcat(command_line_dir,argv[i], NULL);
			if (!strcmp(argv[i],"-debug"))
				{
				debug_mode = TRUE;
				debug_skin = TRUE;
				printf("debugging enabled\n");
				}
			else if (check_for_ipc_command(argv[i]))
				{
				return 0;
				}
			else if (!strcmp(argv[i],"-skinhelp"))
				{
				debug_skin = TRUE;
				printf("skin debugging enabled\n");
				}
			else if (!strncmp(argv[i],"-skin:", 6))
				{
				skin_file = argv[i] + 6;
				if (isdir(skin_file))
					{
					skin_file = strdup(argv[i] + 6);
					printf ("Using skin: %s\n", skin_file);
					}
				else
					{
					gchar *ns = g_strconcat(command_line_dir, "/",  skin_file, NULL);
					if (isdir(ns))
						{
						skin_file = ns;
						printf ("Using skin: %s\n", skin_file);
						}
					else
						{
						g_free(ns);
						ns = g_strconcat(homedir(),
								"/.gqmpeg/skins/", skin_file, NULL);
						if (isdir(ns))
							{
							skin_file = ns;
							printf ("Using skin: %s\n", skin_file);
							}
						else
							{
							g_free(ns);
							printf ("Skin not found: %s\n", skin_file);
							skin_file = NULL;
							}
						}
					}
				}
			else if (isdir(command_line_path) || isdir(argv[i]))
				{
				if (isdir(argv[i]))
					set_current_dir(argv[i]);
				else
					set_current_dir(command_line_path);
				}
			else if (isfile(command_line_path) || isfile(argv[i]))
				{
				if (isfile(argv[i]))
					g_free(command_line_path);
					command_line_path = g_strdup(argv[i]);
				l = strlen(command_line_path);
				if (l > 7)
					l -= 7;
				else
					l = 0;
				if (!strncasecmp(command_line_path + l, ".gqmpeg",7))
					{
					printf("Loading playlist:%s\n",command_line_path);
					playlist_load(command_line_path, FALSE);
					}
				else
					{
					printf("Adding song:%s\n",command_line_path);
					playlist_add_item(command_line_path);
					playlist_song = playlist_count - 1;
					set_song_info(playlist_song);
					}
				}
			else
				{
				printf("unknown option: %s\n",argv[i]);
				}
			g_free(command_line_path);
			i++;
			}
		}

	g_free(command_line_dir);

/* ----- */

	set_song_info(playlist_song);

	/* set up main window */
	mainwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_signal_connect (GTK_OBJECT (mainwindow), "delete_event",(GtkSignalFunc) destroy, NULL);
	gtk_signal_connect (GTK_OBJECT (mainwindow), "key_press_event",(GtkSignalFunc) key_pressed, NULL);
	gtk_window_set_policy (GTK_WINDOW (mainwindow),FALSE,FALSE,TRUE);
	gtk_container_border_width (GTK_CONTAINER (mainwindow), 0);
	gtk_window_set_title (GTK_WINDOW (mainwindow), "GQmpeg");
	gtk_window_set_wmclass(GTK_WINDOW (mainwindow), "GQmpeg", "gqmpeg");

	gtk_widget_realize(mainwindow);
	set_icon_for_window(mainwindow);

	/* set up display */
	display_area = gtk_drawing_area_new();
	gtk_container_add (GTK_CONTAINER (mainwindow), display_area);

	setup_display();

	gtk_widget_show(display_area);

	gtk_widget_realize(display_area);

	dnd_set_drop(display_area, display_area->window);

	/* create pop-up menus */
	main_menu = create_main_menu();
	dir_menu = create_dir_menu();
	file_menu = create_file_menu();
	file_menu_list = create_file_menu_list();
	playlist_menu = create_playlist_menu();

	if (default_skin)
		default_skin_path = g_strconcat(homedir(),
			"/.gqmpeg/skins/", default_skin, NULL);

	if (skin_file)
		{
		printf("loading skin..\n");
		if (!change_to_skin(skin_file))
			{
			if (default_skin_path)
				{
				if (!change_to_skin(default_skin_path))
					{
					printf("Default skin not loaded, using built-in\n");
					change_to_skin(NULL);
					}
				}
			else
				{
				printf("Skin not loaded, using built-in\n");
				change_to_skin(NULL);
				}
			}
		}
	else if (default_skin_path)
		{
		if (!change_to_skin(default_skin_path))
			{
			printf("Default skin not loaded, using built-in\n");
			change_to_skin(NULL);
			}
		}
	else
		change_to_skin(NULL);

	if (default_skin_path) g_free(default_skin_path);

	if (remember_window_position && alternate_skin)
		mode_button_pressed();

	if (!wm_decorations) gdk_window_set_decorations (mainwindow->window, 0);

	gtk_widget_show(mainwindow);

	if (remember_window_position)
		{
		/* ensure that window is at least visible */
		if (window_pos_x < 0 - skin->width)
			window_pos_x = 0;
		if (window_pos_x > gdk_screen_width())
			window_pos_x = gdk_screen_width() - skin->width;
		if (window_pos_y < 0 - skin->height)
			window_pos_y = 0;
		if (window_pos_y > gdk_screen_height())
			window_pos_y = gdk_screen_height() - skin->height;
		gdk_window_move(mainwindow->window, window_pos_x, window_pos_y);
		}

	update_song_count(playlist_count);
	update_song_number(playlist_song);
	set_song_info(playlist_song);
	redraw_name(get_song_title(playlist_song));


	sbuf = g_strconcat("GQmpeg - ", playlist_filename, NULL);
	gtk_window_set_title (GTK_WINDOW (mainwindow), sbuf);
	g_free(sbuf);

/* ----- */

	rebuild_filter();
	read_dir();
	dirlist_sort();
	filelist_sort();

	if (show_playlist_on_startup) show_playlist();

	update_repeat_display();
	update_shuffle_display();
	redraw_display();

	update_timeout_id = gtk_timeout_add(125, (GtkFunction)update_condition, NULL);

	if (start_playing_on_startup) play_button_pressed();

	if (allow_ipc) ipc_on();

	gtk_main ();
	return 0;
}
