/*###################################################################*/
/*##                          GQmpeg 0.4.5                         ##*/
/*##    This software is Copyright (C) 1998, 1999 by John Ellis.   ##*/
/*## This software falls under the GNU Public License. Please read ##*/
/*##              the COPYING file for more information            ##*/
/*###################################################################*/

#include "gqmpeg.h"

static GtkWidget *preset_menu = NULL;

static void load_preset_from_menu(GtkWidget *widget, gpointer data)
{
	gchar *file = data;
	if (!file) return;
	if (load_playlist_from_file(file) == TRUE)
		{
		if (play_presets) start_playing_file();
		}
}

void create_preset_menu()
{
	GtkWidget *menu;
	GtkWidget *item;
	gint i;
        menu = gtk_menu_new();
        for (i=0; i<10; i++)
                {
		gchar s[48];

                if (preset_name[i] && preset_file[i])
                        {
			sprintf(s,"%d - %s", i + 1, preset_name[i]);
			item = gtk_menu_item_new_with_label(s);
			gtk_signal_connect (GTK_OBJECT (item), "activate",
				(GtkSignalFunc) load_preset_from_menu, preset_file[i]);
			}
		else
			{
			sprintf(s,"%d - (empty)", i + 1);
			item = gtk_menu_item_new_with_label(s);
			}
		gtk_menu_append(GTK_MENU(menu),item);
		gtk_widget_show(item);
                }
        gtk_menu_item_set_submenu(GTK_MENU_ITEM(preset_menu),menu);
}

GtkWidget *create_main_menu()
{
	GtkWidget *menu;
	GtkWidget *button;

	menu = gtk_menu_new();

	button = gtk_menu_item_new_with_label("Play");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) play_button_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Pause");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) pause_button_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Stop");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) stop_button_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

        button = gtk_menu_item_new();
        gtk_menu_append(GTK_MENU(menu),button);
        gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Playlist...");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) show_playlist, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	preset_menu = gtk_menu_item_new_with_label("Presets");
	gtk_menu_append(GTK_MENU(menu),preset_menu);
	gtk_widget_show(preset_menu);

        button = gtk_menu_item_new();
        gtk_menu_append(GTK_MENU(menu),button);
        gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Toggle Shuffle");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) shuffle_button_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Toggle Repeat");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) repeat_button_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Toggle Time Display");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) time_button_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

        button = gtk_menu_item_new();
        gtk_menu_append(GTK_MENU(menu),button);
        gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Preferences...");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) show_configuration_window, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Song Info...");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) current_song_info_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Run mixer...");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) mixer_button_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

        button = gtk_menu_item_new();
        gtk_menu_append(GTK_MENU(menu),button);
        gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("About...");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) show_about_info, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

        button = gtk_menu_item_new();
        gtk_menu_append(GTK_MENU(menu),button);
        gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Iconify");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) iconify_button_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Exit");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) destroy, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	create_preset_menu();

	return menu;
}

GtkWidget *create_dir_menu()
{
	GtkWidget *menu;
	GtkWidget *button;

	menu = gtk_menu_new();

	button = gtk_menu_item_new_with_label("Add");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) add_dir_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Add Recursive");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) add_dir_recursive_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	return menu;
}

GtkWidget *create_file_menu()
{
	GtkWidget *menu;
	GtkWidget *button;

	menu = gtk_menu_new();

	button = gtk_menu_item_new_with_label("Add");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) add_button_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Insert");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) insert_button_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

        button = gtk_menu_item_new();
        gtk_menu_append(GTK_MENU(menu),button);
        gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Add and Play");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) add_play_button_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Song Info...");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) filelist_songinfo_pressed, NULL); 
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

        button = gtk_menu_item_new();
        gtk_menu_append(GTK_MENU(menu),button);
        gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Copy");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) filelist_copy_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Move");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) filelist_move_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Rename");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) filelist_rename_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Delete");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) filelist_delete_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	return menu;
}

GtkWidget *create_file_menu_list()
{
	GtkWidget *menu;
	GtkWidget *button;

	menu = gtk_menu_new();

	button = gtk_menu_item_new_with_label("Load");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) load_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Append");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) append_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

        button = gtk_menu_item_new();
        gtk_menu_append(GTK_MENU(menu),button);
        gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Copy");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) filelist_copy_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Move");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) filelist_move_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Rename");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) filelist_rename_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Delete");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) filelist_delete_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	return menu;
}

GtkWidget *create_playlist_menu()
{
	GtkWidget *menu;
	GtkWidget *button;

	menu = gtk_menu_new();

	button = gtk_menu_item_new_with_label("Play");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) pos_play_button_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

        button = gtk_menu_item_new();
        gtk_menu_append(GTK_MENU(menu),button);
        gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Remove");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) remove_button_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Song Info...");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) playlist_songinfo_pressed, NULL); 
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

        button = gtk_menu_item_new();
        gtk_menu_append(GTK_MENU(menu),button);
        gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Copy");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) playlist_copy_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Move");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) playlist_move_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Rename");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) playlist_rename_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	button = gtk_menu_item_new_with_label("Delete");
	gtk_signal_connect (GTK_OBJECT (button), "activate",(GtkSignalFunc) playlist_delete_pressed, NULL);
	gtk_menu_append(GTK_MENU(menu),button);
	gtk_widget_show(button);

	return menu;
}

