/*###################################################################*/
/*##                         GQmpeg 0.4.5                          ##*/
/*##    This software is Copyright (C) 1998, 1999 by John Ellis.   ##*/
/*## This software falls under the GNU Public License. Please read ##*/
/*##              the COPYING file for more information            ##*/
/*###################################################################*/

#include "gqmpeg.h"

typedef struct _Dest_Data Dest_Data;
struct _Dest_Data
{
	GtkWidget *d_clist;
	GtkWidget *f_clist;
	GtkWidget *entry;
	gchar *filter;
};

static void free_dest_data(GtkWidget *widget, gpointer data);
static gint file_passes_filter(gchar *filter, gchar *file);
static gint sort_list_cb(void *a, void *b);
static void populate_dir_clist(Dest_Data *dd, gchar *path);
static void dir_select_cb(GtkWidget *clist, gint row, gint column,
			  GdkEventButton *bevent, gpointer data);

static void free_dest_data(GtkWidget *widget, gpointer data)
{
	Dest_Data *dd = data;
	g_free(dd->filter);
	g_free(dd);
}

static gint file_passes_filter(gchar *filter, gchar *file)
{
        gchar *f_ptr = filter;
        gchar *file_ptr;
        gchar *strt_ptr;
        int i;

        while (f_ptr < filter + strlen(filter))
                {
                strt_ptr = f_ptr;
                i=0;
                while (*f_ptr != ';' && *f_ptr != '\0')
                        {
                        f_ptr++;
                        i++;
                        }
                f_ptr++;
                file_ptr = file + strlen(file) - i;
                if (!strncasecmp(file_ptr, strt_ptr, i)) return TRUE;
                }
        return FALSE;
}

static gint sort_list_cb(void *a, void *b)
{
        return strcmp((gchar *)a, (gchar *)b);
}

static void populate_dir_clist(Dest_Data *dd, gchar *path)
{
        DIR *dp;
        struct dirent *dir;
        struct stat ent_sbuf;
        gchar *buf[] = { "x", };
        GList *path_list = NULL;
        GList *file_list = NULL;
        GList *list;

        if((dp = opendir(path))==NULL)
                {
                /* dir not found */
                return;
                }
        while ((dir = readdir(dp)) != NULL)
                {
                /* skips removed files */
                if (dir->d_ino > 0)
                        {
                        gchar *name = dir->d_name;
			gchar *filepath = g_strconcat(path, "/", name, NULL);
                        if (stat(filepath,&ent_sbuf) >= 0 && S_ISDIR(ent_sbuf.st_mode))
                                {
                                path_list = g_list_insert_sorted(path_list,
                                                g_strdup(name),
                                                (GCompareFunc) sort_list_cb);
                                }
			else if (dd->f_clist)
				{
				if (!dd->filter || (dd->filter && file_passes_filter(dd->filter, name)))
					file_list = g_list_insert_sorted(file_list,
                                                g_strdup(name),
                                                (GCompareFunc) sort_list_cb);
				}
			g_free(filepath);
                        }
		}
        closedir(dp);

	gtk_clist_freeze(GTK_CLIST(dd->d_clist));
	gtk_clist_clear(GTK_CLIST(dd->d_clist));

        list = path_list;
        while (list)
                {
		gint row;
		gchar *filepath;
		if (strcmp(list->data, ".") == 0)
			{
			filepath = g_strdup(path);
			}
		else if (strcmp(list->data, "..") == 0)
			{
			gchar *p;
			filepath = g_strdup(path);
			p = filename_from_path(filepath);
			if (p - 1 != filepath) p--;
			p[0] = '\0';
			}
		else if (strcmp(path, "/") == 0)
			{
			filepath = g_strconcat("/", list->data, NULL);
			}
		else
			filepath = g_strconcat(path, "/", list->data, NULL);
		
		buf[0] = list->data;
		row = gtk_clist_append(GTK_CLIST(dd->d_clist),buf);
		gtk_clist_set_row_data_full(GTK_CLIST(dd->d_clist), row,
					    filepath, (GtkDestroyNotify) g_free);
                g_free(list->data);
                list = list->next;
                }

        g_list_free(path_list);

	gtk_clist_thaw(GTK_CLIST(dd->d_clist));

	if (dd->f_clist)
		{
		gtk_clist_freeze(GTK_CLIST(dd->f_clist));
		gtk_clist_clear(GTK_CLIST(dd->f_clist));

	        list = file_list;
	        while (list)
        	        {
			gint row;
			gchar *filepath;
			filepath = g_strconcat(path, "/", list->data, NULL);
		
			buf[0] = list->data;
			row = gtk_clist_append(GTK_CLIST(dd->f_clist),buf);
			gtk_clist_set_row_data_full(GTK_CLIST(dd->f_clist), row,
					    filepath, (GtkDestroyNotify) g_free);
	                g_free(list->data);
        	        list = list->next;
        	        }

	        g_list_free(file_list);

		gtk_clist_thaw(GTK_CLIST(dd->f_clist));
		}
}

static void dir_select_cb(GtkWidget *clist, gint row, gint column,
			  GdkEventButton *bevent, gpointer data)
{
	Dest_Data *dd = data;
	gchar *path = g_strdup(gtk_clist_get_row_data(GTK_CLIST(clist), row));
	gtk_entry_set_text(GTK_ENTRY(dd->entry),path);

	if (clist == dd->d_clist) populate_dir_clist(dd, path);
	g_free(path);
}

GtkWidget *destination_widget_new_with_files(gchar *path, gchar *filter, GtkWidget *entry)
{
	GtkWidget *vbox;
	Dest_Data *dd;
#ifdef GTK_HAVE_FEATURES_1_1_4
	GtkWidget *scrolled;
#endif

	dd = g_new0(Dest_Data, 1);

	vbox = gtk_vbox_new(FALSE, 5);
	gtk_widget_show(vbox);

	dd->entry = entry;

#ifdef GTK_HAVE_FEATURES_1_1_4
	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
	gtk_widget_show(scrolled);
#endif

	dd->d_clist=gtk_clist_new (1);
        gtk_signal_connect (GTK_OBJECT (dd->d_clist), "select_row",(GtkSignalFunc) dir_select_cb, dd);
	gtk_signal_connect(GTK_OBJECT(dd->d_clist), "destroy", (GtkSignalFunc) free_dest_data, dd);
#ifdef GTK_HAVE_FEATURES_1_1_4
	gtk_container_add (GTK_CONTAINER (scrolled), dd->d_clist);
#else
        gtk_clist_set_policy (GTK_CLIST (dd->d_clist), GTK_POLICY_ALWAYS, GTK_POLICY_AUTOMATIC); 
	gtk_box_pack_start(GTK_BOX(vbox), dd->d_clist, TRUE, TRUE, 0);
#endif
        gtk_widget_show (dd->d_clist);

	if (filter)
		{
#ifdef GTK_HAVE_FEATURES_1_1_4
		scrolled = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
					GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
		gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
		gtk_widget_show(scrolled);
#endif

		dd->f_clist=gtk_clist_new (1);
	        gtk_signal_connect (GTK_OBJECT (dd->f_clist), "select_row",(GtkSignalFunc) dir_select_cb, dd);
#ifdef GTK_HAVE_FEATURES_1_1_4
		gtk_container_add (GTK_CONTAINER (scrolled), dd->f_clist);
#else
        	gtk_clist_set_policy (GTK_CLIST (dd->f_clist), GTK_POLICY_ALWAYS, GTK_POLICY_AUTOMATIC); 
		gtk_box_pack_start(GTK_BOX(vbox), dd->f_clist, TRUE, TRUE, 0);
#endif
	        gtk_widget_show (dd->f_clist);

		dd->filter = g_strdup(filter);
		}

	populate_dir_clist(dd, path);
	return vbox;
}

GtkWidget *destination_widget_new(gchar *path, GtkWidget *entry)
{
	return destination_widget_new_with_files(path, NULL, entry);
}

