/*###################################################################*/
/*##                         GQmpeg 0.4.5                          ##*/
/*##    This software is Copyright (C) 1998, 1999 by John Ellis.   ##*/
/*## This software falls under the GNU Public License. Please read ##*/
/*##              the COPYING file for more information            ##*/
/*###################################################################*/

#include "gqmpeg.h"

/* ----------------------------------------------------------
   input / output interface to mpg123
   ----------------------------------------------------------*/

static FILE *mpgpipe = NULL;
static int mypipe[2];
static gchar pipebuf1[255];
static gchar *pipebuf_ptr;
static gint signal_timeout_id = 0;

static void parse_header_info();
static void parse_frame_info();
static void stop_data();
static void song_finished();
static int parse_pipe_buffer();
static int check_pipe_for_data(int fd);
static gint read_data();

static void parse_header_info()
{
	gchar s[128];
	gchar *ptr;
	gchar *ptr2;

if (debug_mode) printf("h");

	ptr = pipebuf1;

	/* MPEG, layer, freq, mode line */
	if (!strncmp(pipebuf1,"MPEG",4))
		{
if (debug_mode) putchar('M');
		ptr += 4;
		ptr2 = s;
		while (ptr[0] != 'L' && ptr[0] != '\0')
			{
			if (ptr[0] != ' ' && ptr[0] != ',')
				{
				ptr2[0] = ptr[0];
				ptr2++;
				}
			ptr++;
			}
		ptr2[0] = '\0';
		mpeg_version = strtol(s,NULL,0);

		ptr += 6;
		ptr2 = s;
		while (ptr[0] != 'F' && ptr[0] != '\0')
			{
			if (ptr[0] != ' ' && ptr[0] != ',')
				{
				ptr2[0] = ptr[0];
				ptr2++;
				}
			ptr++;
			}
		ptr2[0] = '\0';
		mpeg_layer = strlen(s);

		ptr += 5;
		ptr2 = s;
		while (ptr[0] != 'm' && ptr[0] != '\0')
			{
			if (ptr[0] != ' ' && ptr[0] != ',')
				{
				ptr2[0] = ptr[0];
				ptr2++;
				}
			ptr++;
			}
		ptr2[0] = '\0';
		mpeg_hz = strtol(s,NULL,0);

		ptr += 5;
		ptr2 = s;
		while (ptr[0] != ',' && ptr[0] != '\0')
			{
			if (ptr[0] != ' ')
				{
				ptr2[0] = ptr[0];
				ptr2++;
				}
			ptr++;
			}
		ptr2[0] = '\0';
		if (!strcmp(s,"Stereo")) mpeg_mode = 1;
		if (!strcmp(s,"Joint-Stereo")) mpeg_mode = 2;
		if (!strcmp(s,"Dual-Channel")) mpeg_mode = 3;
		if (!strcmp(s,"Single-Channel")) mpeg_mode = 4;
		}

	/* Channels  line */
	if (!strncmp(pipebuf1,"Channels:",9))
		{
if (debug_mode) putchar('C');
		ptr += 9;
		ptr2 = s;
		while (ptr[0] != ',' && ptr[0] != '\0')
			{
			if (ptr[0] != ' ')
				{
				ptr2[0] = ptr[0];
				ptr2++;
				}
			ptr++;
			}
		ptr2[0] = '\0';
		mpeg_channels = strtol(s,NULL,0);
		}

	/* Bitrate line */
	if (!strncmp(pipebuf1,"Bitrate:",8))
		{
if (debug_mode) putchar('B');
		ptr += 8;
		ptr2 = s;
		while (ptr[0] != 'K' && ptr[0] != '\0')
			{
			if (ptr[0] != ' ')
				{
				ptr2[0] = ptr[0];
				ptr2++;
				}
			ptr++;
			}
		ptr2[0] = '\0';
		mpeg_bitrate = strtol(s,NULL,0);
		}

	/* Audio output line */
	if (!strncmp(pipebuf1,"Audio:",6))
		{
if (debug_mode) putchar('A');
		ptr += 6;
		ptr2 = s;
		while (ptr[0] != ',' && ptr[0] != '\0')
			{
			if (ptr[0] != ' ')
				{
				ptr2[0] = ptr[0];
				ptr2++;
				}
			ptr++;
			}
		ptr2[0] = '\0';
		output_conversion = strtol(s,NULL,0);

		ptr += 7;
		ptr2 = s;
		while (ptr[0] != 'e' && ptr[0] != '\0')
			{
			if (ptr[0] != ' ' && ptr[0] != ',')
				{
				ptr2[0] = ptr[0];
				ptr2++;
				}
			ptr++;
			}
		ptr2[0] = '\0';
		output_hz = strtol(s,NULL,0);

		ptr += 9;
		ptr2 = s;
		while (ptr[0] != ',' && ptr[0] != '\0')
			{
			if (ptr[0] >= '0' && ptr[0] <= '9')
				{
				ptr2[0] = ptr[0];
				ptr2++;
				}
			ptr++;
			}
		ptr2[0] = '\0';
		output_bits = strtol(s,NULL,0);

		ptr += 11;
		ptr2 = s;
		while (ptr[0] != 10 && ptr[0] != '\0')
			{
			if (ptr[0] != ' ')
				{
				ptr2[0] = ptr[0];
				ptr2++;
				}
			ptr++;
			}
		ptr2[0] = '\0';
		output_channels = strtol(s,NULL,0);

		new_song = TRUE;
		}
}

	/* we parse the output to get the current song position
		including seconds and frames (played and remaining) */
static void parse_frame_info()
{
	gchar frame_cnt[12] = "";
	gchar sec_cnt [12] = "";
	gchar frame_cnt_rem [12] = "";
	gchar sec_cnt_rem [12] = "";
	gchar *ptr;
	gchar *ptr2;
	
	if (debug_mode) printf("f");

	ptr = pipebuf1;
	ptr += 6;
	ptr2 = frame_cnt;
	while (ptr[0] != '[' && ptr[0] != '\0')
		{
		if (ptr[0] != ' ')
			{
			ptr2[0] = ptr[0];
			ptr2++;
			}
		ptr++;
		}
	ptr2[0] = '\0';
	ptr++;
	ptr2 = frame_cnt_rem;
	while (ptr[0] != ']' && ptr[0] != '\0')
		{
		if (ptr[0] != ' ')
			{
			ptr2[0] = ptr[0];
			ptr2++;
			}
		ptr++;
		}
	ptr2[0] = '\0';
	while (ptr[0] != ':' && ptr[0] != '\0')
		{
		ptr++;
		}
	ptr++;
	ptr2 = sec_cnt;
	while (ptr[0] != '[' && ptr[0] != '\0')
		{
		if (ptr[0] != ' ')
			{
			ptr2[0] = ptr[0];
			ptr2++;
			}
		ptr++;
		}
	ptr2[0] = '\0';
	ptr++;
	ptr2 = sec_cnt_rem;
	while (ptr[0] != ']' && ptr[0] != '\0')
		{
		if (ptr[0] != ' ')
			{
			ptr2[0] = ptr[0];
			ptr2++;
			}
		ptr++;
		}
	ptr2[0] = '\0';

	frames = strtol(frame_cnt,NULL,0);
	frames_remaining = strtol(frame_cnt_rem,NULL,0);
	seconds = strtol(sec_cnt,NULL,0);
	seconds_remaining = strtol(sec_cnt_rem,NULL,0);
	
}

static void stop_data()
{
	if (signal_timeout_id)
		{
		gtk_timeout_remove(signal_timeout_id);
		signal_timeout_id = 0;
		}
	if (mpgpipe)
		{
		fclose (mpgpipe);
		mpgpipe = NULL;
		}
if (debug_mode) printf("closed\n");
}

static void song_finished()
{
	stop_data();
	status = STATUS_NEXT;
	frames = 0;
}

/* this routine parses the output from mpg123 to check it's current status
   when the end is detected, 1 or TRUE is returned; any error return -1;
   a detected frame (second & frame info) return 1 or TRUE; 2 is returned
   on anything else (including header info) */
static int parse_pipe_buffer()
{
	/* check for Frame data */
	if (!strncmp(pipebuf1,"Frame",5))
		{
		parse_frame_info();
		return 1;
		}

	/* check for end of song (mpg123 reports a line starting
		with a '[') */
	if (!strncmp(pipebuf1,"[",1))
		{
		song_finished();
		waitpid (pid, NULL, 0);
		return 0;
		}

	/* we start looking for mpg123 errors here! */

	/* mpg123 couldn't open device */
	if (!strncmp(pipebuf1,"Can't open",10))
		{

		printf("Error opening output device (mpg123 reported:\"%s\")\n",pipebuf1);

		stop_data();
		waitpid (pid, NULL, 0);
		status = STATUS_PAUSE;
		return -1;
		}

	parse_header_info();
	return 2;
}

	/* copied from the Unix programming FAQ */
static int check_pipe_for_data(int fd)
     {
	int rc;
	fd_set fds;
	struct timeval tv;
	
	FD_ZERO(&fds);
	FD_SET(fd,&fds);
	tv.tv_sec = tv.tv_usec = 0;

	rc = select(fd+1, &fds, NULL, NULL, &tv);
	if (rc < 0)
		return -1;

	return FD_ISSET(fd,&fds) ? 1 : 0;
}


static gint read_data()
{
	int c, d;
	static int dcheck;

	if (debug_mode) printf("I");

	if (status != STATUS_PLAY) return FALSE;
	
	if (!mpgpipe)
		{
		mpgpipe = fdopen (mypipe[0], "r");

		if (debug_mode) printf("pipe opened\n");

		setvbuf(mpgpipe,NULL,_IONBF,BUFSIZ);
		pipebuf_ptr = pipebuf1;
		}

	/* if mpg123 starts sending too much data, we exit this loop
		after 8k bytes so the program will respond to user
		input. Otherwise, the program will appear hung.*/
	d = 8000;

	while (d > 0 && check_pipe_for_data(mypipe[0]))
		{
		c = getc(mpgpipe);
		if (c == 13 || c == 10)
			{
			pipebuf_ptr[0] = '\0';
/*			if (TRUE) printf("%s\n",pipebuf1);*/
			if (parse_pipe_buffer() == -1) return FALSE;
			if (status == STATUS_NEXT) return FALSE;
			pipebuf_ptr = pipebuf1;
			}
		else
			{
			pipebuf_ptr[0] = c;
			pipebuf_ptr++;
			}
		d--;
		}

	/* here we check to see if mpg123 sent data within the last
		10 seconds (68 times through [at 150ms per] with no
		data). no data means mpg123 encountered an error we
		did not catch or it crashed */
	if ( d == 8000)
		dcheck++;
	else
		dcheck = 0;
	if (dcheck > 68)
		{
		printf("Error: mpg123 stopped sending data!\n");
		song_finished();
		waitpid (pid, NULL, 0);
		return FALSE;
		}

	if (debug_mode) printf("O");

	return TRUE;
}

void start_playing_file()
{
	pid_t frk_pid;
	char cmd_arguments[16][512];
	char *cmd_ptr[16];
	int cmd_cnt = 0;

	if (status == STATUS_PLAY) return;

	/* create all command line arguments */

	strcpy(cmd_arguments[cmd_cnt],"mpg123");
	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
	cmd_cnt++;

	strcpy(cmd_arguments[cmd_cnt],"-v");
	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
	cmd_cnt++;

	strcpy(cmd_arguments[cmd_cnt],"-k");
	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
	cmd_cnt++;
	sprintf(cmd_arguments[cmd_cnt],"%d",frames);
	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
	cmd_cnt++;

	if (mpg123_buffer_enable)
		{
		strcpy(cmd_arguments[cmd_cnt],"-b");
		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
		cmd_cnt++;
		sprintf(cmd_arguments[cmd_cnt],"%d",mpg123_buffer);
		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
		cmd_cnt++;
		}

	if (mpg123_downsample != MPG123_DOWNSAMPLE_AUTO)
		{
		if (mpg123_downsample == MPG123_DOWNSAMPLE_22)
			{
			strcpy(cmd_arguments[cmd_cnt],"-2");
			cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
			cmd_cnt++;
			}
		if (mpg123_downsample == MPG123_DOWNSAMPLE_11)
			{
			strcpy(cmd_arguments[cmd_cnt],"-4");
			cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
			cmd_cnt++;
			}
		if (mpg123_downsample == MPG123_DOWNSAMPLE_CUSTOM)
			{
			sprintf(cmd_arguments[cmd_cnt],"-r %d", mpg123_custom_sample_size);
			cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
			cmd_cnt++;
			}
		}

	if (mpg123_mono)
		{
		strcpy(cmd_arguments[cmd_cnt],"-m");
		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
		cmd_cnt++;
		}

	if (mpg123_8bit)
		{
		strcpy(cmd_arguments[cmd_cnt],"--8bit");
		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
		cmd_cnt++;
		}

	if (mpg123_device_enable && mpg123_device)
		{
		strcpy(cmd_arguments[cmd_cnt],"-a");
		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
		cmd_cnt++;
		strcpy(cmd_arguments[cmd_cnt],mpg123_device);
		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
		cmd_cnt++;
		}

	strcpy(cmd_arguments[cmd_cnt],song_path);
	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
	cmd_cnt++;

	strcpy(cmd_arguments[cmd_cnt], "");
	cmd_ptr[cmd_cnt] = NULL;
	cmd_cnt++;


	  /* Create the pipe. */
	if (pipe (mypipe))
		{
		fprintf (stderr, "Pipe failed.\n");
		return;
		}

	if (debug_mode) printf("opening: %s\n",song_path);

	/* Create the child process. */
	frk_pid = fork ();
	if (frk_pid == (pid_t) 0)
		{
		/* This is the child process. */
		dup2(mypipe[1],2);
		close(mypipe[0]);

		/* set the group (session) id to this process for future killing */
		setsid();

		execvp("mpg123",cmd_ptr);
		exit(1);
		}
	else if (frk_pid < (pid_t) 0)
		{
		/* The fork failed. */
		fprintf (stderr, "Fork failed.\n");
		pid = 0;
		return;
		}
	else
		{
		/* This is the parent process. */
		pid = (int) frk_pid;
		close(mypipe[1]);
		}
	status = STATUS_PLAY;
	signal_timeout_id = gtk_timeout_add(150,(GtkFunction) read_data,NULL);
}

void stop_playing_file()
{
	if (status == STATUS_STOP) return;
	stop_data();

	/* kill the entire mpg123 group to work around mpg123 buffer bug */
	kill(-pid, SIGTERM);

	waitpid (pid, NULL, 0);
}

