/*###################################################################*/
/*##                         GQmpeg 0.4.5                          ##*/
/*##    This software is Copyright (C) 1998, 1999 by John Ellis.   ##*/
/*## This software falls under the GNU Public License. Please read ##*/
/*##              the COPYING file for more information            ##*/
/*###################################################################*/

#include "gqmpeg.h"

static GtkWidget *dirlist_clist;
static GtkWidget *filelist_clist;
static GtkWidget *playlist_clist;
static GtkWidget *pathentry;
static GtkWidget *historypulldown;
static GtkWidget *fileinfolabel;

static GList *shuffle_list;

static gint dir_pos;

static GList *shuffle_list_find_node(gint p);
static void shuffle_list_add (gint p);
static void shuffle_list_remove (gpointer data);
static void playlist_free_data(Song_Data *d);
static Song_Data *playlist_new_data(gchar *path);
static void playlist_add_dir_func(gchar *d, gint recursive);
static void playlist_add_dir(gchar *d);
static void playlist_add_dir_recursive(gchar *d);
static void save_pressed();
static void save_as_pressed();
static void save_mode_changed(GtkWidget *w, gpointer data);
static void start_playing_song_number(gint n);
static void playlist_item_selected(GtkWidget *widget, GdkEventButton *bevent, gpointer data);
static void change_dir_to_home();
static void change_dir(GtkWidget *widget, GdkEventButton *bevent, gpointer data);
static void change_file(GtkWidget *widget, GdkEventButton *bevent, gpointer data);
static void add_all_button_pressed();
static void remove_all_button_pressed();
static void up_button_pressed();
static void down_button_pressed();
static void top_button_pressed();
static void end_button_pressed();
static void path_entry_select(gchar *newdir, gpointer data);
static void history_menu_select(GtkWidget *widget, gchar *newdir);
static gchar *truncate_hist_text(gchar *t, gint l);
static void update_history_menu();
static void populate_playlist_clist();
static void hide_playlist();
static void create_playlist_window();


void shuffle_list_create(gint start_with_current)
{
	GList *work = NULL;
	GList *node = NULL;
	int i;

	if (shuffle_list)
		shuffle_list_destroy();

	if (debug_mode) printf("Shuffling:");

	if (playlist_count == 0) return;
		
	for (i=0; i < playlist_count; i++)
		{
		if (i != playlist_song || !start_with_current)
			work = g_list_append(work, playlist_get_data(i));
		}

	if (start_with_current && playlist_song >= 0 && playlist_song < playlist_count)
		{
		node = g_list_nth(playlist, playlist_song);
		shuffle_list = g_list_append(shuffle_list, node->data);
		if (debug_mode) printf("%d ", playlist_song);
		}
	else if (playlist_count > 1 && playlist_song >= 0 && playlist_song < playlist_count)
		{
		gint p;

		/* remove current song so we can not pick it */
		node = g_list_nth(work, playlist_song);
		g_list_remove(work, node->data);

		p = (float)rand() / RAND_MAX * g_list_length(work);
		node = g_list_nth(work, p);
		if (debug_mode) printf("%d ",g_list_index(playlist, node->data));
		shuffle_list = g_list_append(shuffle_list, node->data);

		/* and put current song back in*/
		g_list_insert(work, playlist_get_data(playlist_song), playlist_song);

		work = g_list_remove(work, node->data);
		}

	while (work)
		{
		gint p =  (float)rand() / RAND_MAX * g_list_length(work);
		node = g_list_nth(work, p);
		if (debug_mode) printf("%d ",g_list_index(playlist, node->data));
		shuffle_list = g_list_append(shuffle_list, node->data);
		work = g_list_remove(work, node->data);
		}

	if (debug_mode) printf ("\n");
}

void shuffle_list_destroy()
{
	if (debug_mode) printf("Clearing shuffle list\n");
	if (!shuffle_list) return;
	g_list_free(shuffle_list);
	shuffle_list = NULL;
}

gint shuffle_list_get_first()
{
	return g_list_index(playlist, shuffle_list->data);
}

gint shuffle_list_get_next(gint p)
{
	GList *node;

	if (!shuffle_list) return p;

	node = shuffle_list_find_node(p);
	node = node->next;

	if (!node) return p;

	return g_list_index(playlist, node->data);
}

gint shuffle_list_get_prev(gint p)
{
	GList *node;

	if (!shuffle_list) return p;

	node = shuffle_list_find_node(p);
	node = node->prev;

	if (!node) return p;

	return g_list_index(playlist, node->data);
}

static GList *shuffle_list_find_node(gint p)
{
	GList *work = g_list_nth(playlist, p);
	return g_list_find(shuffle_list, work->data);
}

static void shuffle_list_add (gint p)
{
	gint shuffle_p = (float)rand() / RAND_MAX * ( g_list_length(shuffle_list) + 1);
	GList *node = g_list_nth(playlist, p);
	if (shuffle_p == g_list_length(shuffle_list))
		shuffle_list = g_list_append(shuffle_list, node->data);
	else
		shuffle_list = g_list_insert(shuffle_list, node->data, shuffle_p);
}

static void shuffle_list_remove (gpointer data)
{
	shuffle_list = g_list_remove(shuffle_list, data);
}


void update_list_highlight(GtkWidget *clistw, gint old, gint new, gint move)
{
	if (old >= 0)
		{
#ifdef GTK_HAVE_FEATURES_1_1_8
		gtk_clist_set_background(GTK_CLIST(clistw), old,
			&GTK_WIDGET (clistw)->style->base[GTK_STATE_NORMAL]);
		gtk_clist_set_foreground(GTK_CLIST(clistw), old,
			&GTK_WIDGET (clistw)->style->fg[GTK_STATE_NORMAL]);
#else
		gtk_clist_set_background(GTK_CLIST(clistw), old,
			&GTK_WIDGET (clistw)->style->bg[GTK_STATE_PRELIGHT]);
		gtk_clist_set_foreground(GTK_CLIST(clistw), old,
			&GTK_WIDGET (clistw)->style->fg[GTK_STATE_PRELIGHT]);
#endif
		}
	if (new >= 0)
		{
		gtk_clist_set_background(GTK_CLIST(clistw), new,
			&GTK_WIDGET (clistw)->style->bg[GTK_STATE_SELECTED]);
		gtk_clist_set_foreground(GTK_CLIST(clistw), new,
			&GTK_WIDGET (clistw)->style->fg[GTK_STATE_SELECTED]);
		}
	if (move)
		gtk_clist_moveto (GTK_CLIST(clistw), new, 0, 0.5, 0.0);
}

/* returns FALSE if can't access file, TRUE if a playlist,
	(-1) otherwise (invalid playlist)*/
gint playlist_load(gchar *fn, gint append)
{
	gchar s_buf[1025];
	gchar *s_buf_ptr;
	FILE *f;

	f = fopen (fn,"r");
	if (!f)
		{
		/* file open failed */
		printf("failed to open %s\n",fn);
		return FALSE;
		}
	else
		{
		/* check first line for valid playlist */
		fgets(s_buf,1024,f);
		if (strncmp(s_buf,"# GQmpeg",8) != 0)
			{
			/* we reach here, file is not a valid playlist */
			fclose (f);
			return -1;
			}

		/* load playlist */
		if (!append) playlist_clear();
		while (fgets(s_buf,1024,f))
			{
			if (s_buf[0]=='#')
				{
				if (!append && obey_mode_in_playlist && strncmp(s_buf,"# Shuffl",8) == 0)
					{
					gint old_shuffle = shuffle_mode;
					gint old_repeat = repeat_mode;
					if (sscanf(s_buf, "# Shuffle: %d Repeat: %d",
							&shuffle_mode, &repeat_mode) > 0)
						{
						if (old_shuffle != shuffle_mode)
							{
							if (shuffle_mode)
								shuffle_list_create(FALSE);
							else
								shuffle_list_destroy();
							update_shuffle_display();
							}
						if (old_repeat != repeat_mode)
							update_repeat_display();
						}

					}
				continue;
				}
			if (s_buf[0]=='\n') continue;
			s_buf_ptr = s_buf;
			while (s_buf_ptr[0] != '\n' && s_buf_ptr[0] != '\0' && s_buf_ptr < s_buf + strlen(s_buf)) s_buf_ptr++;
			s_buf_ptr[0] = '\0';
			if (isfile(s_buf)) playlist_add_item(s_buf);
			}
		fclose (f);
		}

	if (!append)
		{
		if (playlist_pathname) g_free(playlist_pathname);
		playlist_pathname = g_strdup(fn);
		if (playlist_filename) g_free(playlist_filename);
		playlist_filename = g_strdup(filename_from_path(playlist_pathname));

		if (playlist_count > 0)
			{
			if (shuffle_mode)
				{
				playlist_song = shuffle_list_get_first();
				}
			else
				playlist_song = 0;
			set_song_info(playlist_song);

			if (display_area)
				{
				update_song_number(playlist_song);
				redraw_name(get_song_title(playlist_song));
				}
			}
		else
			{
			playlist_song = -1;
			set_song_info(playlist_song);

			if (display_area)
				{
				update_song_number(-1);
				redraw_name(get_song_title(playlist_song));
				}
			}
		}

	if (display_area)
		{
		update_song_count(playlist_count);
		redraw_display();
		}

	return TRUE;
}

gint playlist_save(gchar *fn)
{
	int i;
	FILE *f;

	f = fopen (fn,"w");
	if (!f)
		{
		/* file open failed */
		printf("failed to open %s\n",fn);
		return FALSE;
		}

	fprintf(f,"# GQmpeg song playlist\n");
	fprintf(f,"# created by version %d.%d.%d\n",GQMPEG_VERSION_MAJOR,GQMPEG_VERSION_MINOR,GQMPEG_VERSION_REV);

	if (save_mode_in_playlist)
		{
		fprintf(f,"# Shuffle:%d Repeat:%d (please only use 0 or 1)\n", shuffle_mode, repeat_mode);
		}

	for (i=0; i < playlist_count; i++)
		{
		fprintf(f,"%s\n",playlist_get_item(i));
		}

	fprintf(f,"# end of list #\n");
	fclose (f);
	return TRUE;
}

void playlist_init()
{
	playlist = NULL;
	playlist_count = 0;
	playlist_pos = -1;
	playlist_length = 0;
}

static void playlist_free_data(Song_Data *d)
{
	if (d->tag) free_id3_tag(d->tag);
	g_free(d->hdr);
	g_free(d->path);
	g_free(d);
}

static Song_Data *playlist_new_data(gchar *path)
{
	Song_Data *d;

	d = g_new0(Song_Data, 1);
	if (path)
		d->path = g_strdup(path);
	if (read_id3_tags)
		d->tag = get_id3_tag(path);
	if (playlist_accounting)
		d->hdr = get_mpg_header_info(path);

	return d;
}

void playlist_clear()
{
        if (playlist)
                {
                GList *list;
                list = playlist;
                while (list)
                        {
                        playlist_free_data(list->data);
                        list = list->next;
                        }
                g_list_free(playlist);
                playlist = NULL;
                }
	playlist_init();

	if (shuffle_mode) shuffle_list_destroy();
}

void playlist_add_item(gchar *fn)
{
	Song_Data *d = playlist_new_data(fn);
	playlist = g_list_append(playlist, d);
	playlist_count++;
	if (d->hdr) playlist_length += d->hdr->length;

	if (shuffle_mode) shuffle_list_add (g_list_index(playlist, d));
}

void playlist_insert_item(gchar *fn, gint n)
{
	Song_Data *d = playlist_new_data(fn);

	if (n >= playlist_count)
		playlist = g_list_append(playlist, d);
	else
		playlist = g_list_insert(playlist, d, n);

	playlist_count++;
	if (d->hdr) playlist_length += d->hdr->length;

	if (shuffle_mode) shuffle_list_add (g_list_index(playlist, d));
}

void playlist_remove_item(gint n)
{
	Song_Data *d;
	GList *list;
	if (n >= playlist_count) return;
	list = g_list_nth(playlist,n);

	d = list->data;

	if (shuffle_mode) shuffle_list_remove (d);

	playlist = g_list_remove(playlist,d);
	playlist_count--;
	if (d->hdr) playlist_length -= d->hdr->length;

	playlist_free_data(d);
}

void playlist_swap_items(gint n1, gint n2)
{
	GList *list1;
	GList *list2;
	Song_Data *d1;
	Song_Data *d2;

	if (n1 >= playlist_count || n2 >= playlist_count) return;

	list1 =  g_list_nth(playlist,n1);
	list2 =  g_list_nth(playlist,n2);

	d1 = list1->data;
	d2 = list2->data;

	list1->data = d2;
	list2->data = d1;
}

gchar *playlist_get_item(gint n)
{
	Song_Data *d;
	GList *list;
	if (n >= playlist_count) return NULL;
	list = g_list_nth(playlist, n);
	d = list->data;
	return d->path;
}

gchar *playlist_get_title(gint n)
{
	Song_Data *d;
	GList *list;
	if (n >= playlist_count) return NULL;
	list = g_list_nth(playlist, n);
	d = list->data;

	if (d->tag && d->tag->title)
		return d->tag->title;
	else
		return filename_from_path(d->path);
}

Song_Data *playlist_get_data(gint n)
{
	GList *list;
	if (n >= playlist_count) return NULL;
	list = g_list_nth(playlist, n);
	return list->data;
}

gint playlist_get_index(gchar *fn)
{
	Song_Data *d;
	GList *list = playlist;
	gint c = 0;

	while (list)
		{
		d = list->data;
		if (strcmp(d->path, fn) == 0) return c;
		c++;
		list = list->next;
		}

	return -1;
}

void playlist_update_tags()
{
	GList *list;
	if (!playlist) return;

	list = playlist;
	while (list)
		{
		Song_Data *d = list->data;
		if (d->tag)
			free_id3_tag(d->tag);
		if (read_id3_tags)
			d->tag = get_id3_tag(d->path);
		else
			d->tag = NULL;
		list = list->next;
		}

	if (playlist_window)
		populate_playlist_clist();

	redraw_name(playlist_get_title(playlist_song));
}

void playlist_update_headers()
{
	GList *list;
	if (!playlist) return;

	playlist_length = 0;
	list = playlist;
	while (list)
		{
		Song_Data *d = list->data;
		g_free (d->hdr);
		if (playlist_accounting)
			{
			d->hdr = get_mpg_header_info(d->path);
			if (d->hdr) playlist_length += d->hdr->length;
			}
		else
			d->hdr = NULL;
		list = list->next;
		}
}

void set_song(int n)
{
	playlist_song = n;
	set_song_info(playlist_song);
	frames = 0;

	redraw_name(get_song_title(playlist_song));
	update_song_number(playlist_song);
	redraw_display();
}

void playlist_add_song(gchar *fn)
{
	playlist_add_item(fn);
	if (playlist_window)
		{
		gchar *buf[] = { "x", };
		buf[0] = playlist_get_title(playlist_count - 1);
		gtk_clist_append(GTK_CLIST(playlist_clist),buf);
		}

	if (playlist_count == 1)
		{
		playlist_song = 0;
		set_song_info(playlist_song);
		redraw_name(get_song_title(playlist_song));
		update_song_number(playlist_song);
		}
	update_song_count(playlist_count);
	redraw_display();
}
 
void playlist_insert_song(gchar *fn, gint n)
{
	if (n < 0 || n >= playlist_count)
		{
		playlist_add_song(fn);
		return;
		}
	playlist_insert_item(fn, n);

	if (playlist_window)
		{
		gchar *buf[] = { "x", };
		buf[0] = playlist_get_title(n);
		gtk_clist_insert(GTK_CLIST(playlist_clist), n, buf);
		if (n <= playlist_pos) playlist_pos++;
		}

	if (n <= playlist_song)
		{
		playlist_song++;
		update_song_number(playlist_song);
		}
	update_song_count(playlist_count);
	redraw_display();
}

void playlist_remove_song(gchar *fn, gint n)
{
	if (fn)
		{
		n = playlist_get_index(fn);
		if (n < 0) return;
		}

	while (n >= 0)
		{
		playlist_remove_item(n);

		if (playlist_window)
			{
			gtk_clist_remove(GTK_CLIST(playlist_clist),n);
			if (playlist_pos > n) playlist_pos--;
			if (playlist_pos >= playlist_count) playlist_pos = playlist_count - 1;
			update_list_highlight(playlist_clist, playlist_pos, playlist_pos, FALSE);
			}

		if (playlist_count == 0)
			{
			if (status == STATUS_PAUSE) status = STATUS_STOP;
			if (status == STATUS_PLAY)
				{
				stop_playing_file();
				status = STATUS_STOP;
				}
			playlist_song = -1;
			set_song_info(playlist_song);
			redraw_name(get_song_title(playlist_song));
			}
		else if (n < playlist_song)
			{
			playlist_song--;
			update_song_number(playlist_song);
			}
		else if (n == playlist_song)
			{
			if (playlist_song >= playlist_count) playlist_song--;
			if (status == STATUS_PLAY)
				{
				stop_playing_file();
				status = STATUS_STOP;
				play_button_pressed();
				}
			else
				{
				if (status == STATUS_PAUSE) status = STATUS_STOP;
				set_song_info(playlist_song);
				}
			redraw_name(get_song_title(playlist_song));
			}

		if (fn)
			n = playlist_get_index(fn);
		else
			n = -1;
		}
	update_song_count(playlist_count);
	redraw_display();
}

void playlist_replace_song(gchar *old_fn, gchar *new_fn)
{
	gint n = playlist_get_index(old_fn);
	while (n >= 0)
		{
		playlist_remove_item(n);
		playlist_insert_item(new_fn, n);

		if (playlist_window)
			{
			gtk_clist_set_text (GTK_CLIST(playlist_clist), n, 0, playlist_get_title(n));
			}

		if (n == playlist_song)
			{
			if (status == STATUS_PLAY)
				{
				stop_playing_file();
				status = STATUS_STOP;
				set_song_info(n);
				start_playing_file();
				}
			else
				{
				if (status == STATUS_PAUSE) status = STATUS_STOP;
				set_song_info(n);
				}
			redraw_name(get_song_title(n));
			}
		n = playlist_get_index(old_fn);
		}
}

static void playlist_add_dir_func(gchar *d, gint recursive)
{
	DIR *dp;
	struct dirent *dir;
	struct stat ent_sbuf;

	if((dp = opendir(d))==NULL)
		{
		/* dir not found */
		return;
		}

	while ((dir = readdir(dp)) != NULL)
		{
		/* skips removed files */
		if (dir->d_ino > 0)
			{
			gchar *name;
			gchar *path;

			name = dir->d_name; 
			path = g_strconcat(d, "/", name, NULL);

			if (stat(path,&ent_sbuf) >= 0 && S_ISDIR(ent_sbuf.st_mode))
				{
				if (dot_file_check(name) || (DOT_FILES))
					if (strcmp(name, ".") != 0 && strcmp(name, "..") != 0)
						if (recursive)
							playlist_add_dir_func(path, TRUE);
				}
			else
				{
				if ( (dot_file_check(name) || DOT_FILES) &&
						(filter_file_check(name) || FILTERING_DISABLE) )
					{
					if (!isplaylist(path))
						{
						playlist_add_item(path);
						if (playlist_window)
							{
							gchar *buf[] = { "x", };
							buf[0] = playlist_get_title(playlist_count - 1);
							gtk_clist_append(GTK_CLIST(playlist_clist),buf);
							}
						}

					}
				}
			g_free(path);
			}
		}

        closedir(dp);
}

static void playlist_add_dir(gchar *d)
{
	playlist_add_dir_func(d, FALSE);
	update_song_count(playlist_count);
	redraw_display();
}

static void playlist_add_dir_recursive(gchar *d)
{
	playlist_add_dir_func(d, TRUE);
	if (display_area)
	update_song_count(playlist_count);
	redraw_display();
}

void add_dir_pressed()
{
	gchar *path;
	if (dir_pos < 0 || dir_pos >= dirlist_count) return;
	path = g_strconcat(current_dir, "/", dirlist_get_item(dir_pos), NULL);
	playlist_add_dir(path);
	g_free(path);
}

void add_dir_recursive_pressed()
{
	gchar *path;
	if (dir_pos < 0 || dir_pos >= dirlist_count) return;
	path = g_strconcat(current_dir, "/", dirlist_get_item(dir_pos), NULL);
	playlist_add_dir_recursive(path);
	g_free(path);
}

#ifdef __GTK_DND_H__

#else
void dnd_data_dropped(GtkWidget *widget, GdkEventDropDataAvailable *event)
{
	int count = event->data_numbytes;
	char *ptr = event->data;
	int l;
	int old_count = playlist_count;

	while (count > 0)
		{
		l = strlen(ptr) + 1;
		if (isfile(ptr))
			{
			if (filter_file_check(ptr))
				{
				if (isplaylist(ptr))
					{
					if (playlist_load(ptr, FALSE))
						{
						count = 0;
						old_count = 0;
						if (playlist_window)
							{
							gchar *sbuf = g_strconcat("GQmpeg Playlist - ", playlist_filename, NULL);
							gtk_window_set_title (GTK_WINDOW (playlist_window), sbuf);
							g_free(sbuf);
							populate_playlist_clist();
							}
						}
					}
				else
					playlist_add_song(ptr);
				}
			}
		if (isdir(ptr)) playlist_add_dir(ptr);
		ptr += l;
		count -= l;
		}
	if (old_count < playlist_count)
		{
		if (status == STATUS_PAUSE) status = STATUS_STOP;
		if (status == STATUS_PLAY)
			{
			stop_playing_file();
			status = STATUS_STOP;
			}
		set_song(old_count);
		start_playing_file();
		}
	if (old_count != playlist_count)
		{
		update_song_count(playlist_count);
		redraw_display();
		}
}
#endif

static void save_pressed()
{
	if (isfile(playlist_pathname) && strcmp(playlist_pathname,"") !=0)
		{
		playlist_save(playlist_pathname);
		}
	else
		{
		playlist_save_as_dialog(playlist_pathname);
		}
}

static void save_as_pressed()
{
	playlist_save_as_dialog(playlist_pathname);
}

static void save_mode_changed(GtkWidget *w, gpointer data)
{
	save_mode_in_playlist = GTK_TOGGLE_BUTTON(w)->active;
}

gint load_playlist_from_file(gchar *fn)
{
	gint load_val;
	gchar *sbuf;
	load_val = playlist_load(fn, FALSE);
	if (load_val == TRUE)
		{
		sbuf = g_strconcat("GQmpeg - ",playlist_filename, NULL);
		gtk_window_set_title (GTK_WINDOW (mainwindow), sbuf);
		g_free(sbuf);
		if (playlist_window)
			{
			sbuf = g_strconcat("GQmpeg Playlist - ", playlist_filename, NULL);
			gtk_window_set_title (GTK_WINDOW (playlist_window), sbuf);
			g_free(sbuf);
			populate_playlist_clist();
			}

		/* if playing, change to first song of new playlist */
		if (status == STATUS_PLAY)
			{
			stop_playing_file();
			status = STATUS_STOP;
			frames = 0;
			/* only start playing if playlist contains a song */
			if (playlist_count > 0)
				{
				start_playing_file();
				}
			}
		}
	if (load_val == FALSE)
		{
		warning_dialog("Open failed","The specified file could\nnot be opened.");
		}
	if (load_val == -1)
		{
		warning_dialog("Invalid file","The specified file is\nnot a valid gqmpeg playlist.");
		}

	return load_val;
}

gint append_playlist_from_file(gchar *fn)
{
	gint load_val = playlist_load(fn, TRUE);
	if (load_val == TRUE)
		{
		if (playlist_window) populate_playlist_clist();
		}
	return load_val;
}

void load_pressed()
{
	load_playlist_from_file(current_path);
}

void append_pressed()
{
	append_playlist_from_file(current_path);
}

void load_button_pressed()
{
	gchar *buf = g_strconcat(homedir(), "/.gqmpeg/playlists/", NULL);
	playlist_load_dialog(buf);
	g_free(buf);
}

void append_button_pressed()
{
	gchar *buf = g_strconcat(homedir(), "/.gqmpeg/playlists/", NULL);
	playlist_append_dialog(buf);
	g_free(buf);
}

static void start_playing_song_number(gint n)
{
	if (n < 0 || n > playlist_count - 1) return;

	if (status == STATUS_PAUSE)
		{
		status = STATUS_STOP;
		}

	if (status == STATUS_PLAY)
		{
		stop_playing_file();
		status = STATUS_STOP;
		}

	playlist_song = n;
	set_song_info(playlist_song);
	start_playing_file();
	redraw_name(get_song_title(playlist_song));
	update_song_number(playlist_song);
	redraw_display();		
}

static void playlist_item_selected(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
{
	gint row = -1;
	gint column = -1;
	if (bevent->window != GTK_CLIST(widget)->clist_window) return;

	gtk_clist_get_selection_info (GTK_CLIST (widget), bevent->x, bevent->y, &row, &column);
	if (row == -1 || column == -1) return;

	if (debug_mode)	printf("%s\n",playlist_get_item(row));

	gtk_clist_unselect_row(GTK_CLIST(playlist_clist),row,0);

	if (bevent->button == 1 || bevent->button == 2 || bevent->button == 3)
		{
		update_list_highlight(playlist_clist, playlist_pos, row, FALSE);
		playlist_pos = row;
		/* double click to start playing song */
		if (bevent->button == 1 && bevent->type == GDK_2BUTTON_PRESS)
			start_playing_song_number(playlist_pos);
		if (bevent->button == 2)
			start_playing_song_number(playlist_pos);
		if (bevent->button == 3)
			gtk_menu_popup (GTK_MENU(playlist_menu), NULL, NULL, NULL, NULL,
				bevent->button, bevent->time);
		}
}

static void change_dir_to_home()
{
	char *newdir;
	newdir = homedir();
	set_current_dir(newdir);
	refresh_dir();
}

static void change_dir(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
{
	gchar *newdir;
	gint row = -1;
	gint column = -1;
	if (bevent->window != GTK_CLIST(widget)->clist_window) return;

	gtk_clist_get_selection_info (GTK_CLIST (widget), bevent->x, bevent->y, &row, &column);
	if (row == -1 || column == -1) return;

	gtk_clist_unselect_row(GTK_CLIST(widget),row,0);

	if (bevent->button == 1 && bevent->type == GDK_2BUTTON_PRESS)
		{
		newdir = dirlist_get_item(row);
		set_current_dir(newdir);
		if (current_file) g_free(current_file);
		current_file = NULL;
		refresh_dir();
		}
	if (bevent->button == 3)
		{
		dir_pos = row;
		gtk_menu_popup (GTK_MENU(dir_menu), NULL, NULL, NULL, NULL,
				bevent->button, bevent->time);
		
		}
}

static void change_file(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
{
	gint row = -1;
	gint column = -1;
	if (bevent->window != GTK_CLIST(widget)->clist_window) return;

	gtk_clist_get_selection_info (GTK_CLIST (widget), bevent->x, bevent->y, &row, &column);
	if (row == -1 || column == -1) return;

	gtk_clist_unselect_row(GTK_CLIST(widget),row,0);

	if (current_file) g_free(current_file);
	current_file = g_strdup(filelist_get_item(row));
	if (current_path) g_free(current_path);
	current_path = g_strconcat(current_dir, "/", current_file, NULL);

	if (bevent->button == 1 && bevent->type == GDK_2BUTTON_PRESS) add_button_pressed();
	if (bevent->button == 2) add_button_pressed();
	if (bevent->button == 3)
		{
		if (isplaylist(current_path))
			gtk_menu_popup (GTK_MENU(file_menu_list), NULL, NULL, NULL, NULL,
					bevent->button, bevent->time);
		else
			gtk_menu_popup (GTK_MENU(file_menu), NULL, NULL, NULL, NULL,
					bevent->button, bevent->time);
		}

}

void add_button_pressed()
{
	if (!current_file) return;
	/* check to only add if not a playlist file */
	if (strncasecmp(current_file + (strlen(current_file) - 7), ".gqmpeg", 7) == 0) return;

	playlist_add_song(current_path);
}

void remove_button_pressed()
{
	if (playlist_pos < 0) return;
	playlist_remove_song(NULL, playlist_pos);
}

void insert_button_pressed()
{
	if (!current_file) return;
	/* check to only add if not a playlist file */
	if (strncasecmp(current_file + (strlen(current_file) - 7), ".gqmpeg", 7) == 0) return;
	playlist_insert_song(current_path, playlist_pos);
}

static void add_all_button_pressed()
{
	gchar *path_buf;
	gchar *buf[] = { "x", };
	gchar *fn;
	gint row;
	gint i;
	gint playlist_count_remember = playlist_count;
	if (filelist_count < 1) return;
	
	gtk_clist_freeze(GTK_CLIST(playlist_clist));
	for (i=0; i < filelist_count; i++)
		{
		fn = filelist_get_item(i);
		/* check to only add if not a playlist file */
		if (strncasecmp(fn + (strlen(fn) - 7), ".gqmpeg", 7) != 0)
			{
			path_buf = g_strconcat(current_dir, "/", fn, NULL);
			playlist_add_item(path_buf);

			buf[0] = playlist_get_title(playlist_count -1);
			row = gtk_clist_append(GTK_CLIST(playlist_clist),buf);

			g_free(path_buf);
			}
		}
	gtk_clist_thaw(GTK_CLIST(playlist_clist));

	/* if playlist was empty, set current song to the first song we just added */
	if (playlist_count_remember == 0 && playlist_count > 0)
		{
		playlist_song = 0;
		if (shuffle_mode)
			shuffle_list_create(TRUE); /* this is done so the first song is not always last */
		frames = 0;
		set_song_info(playlist_song);
		redraw_name(get_song_title(playlist_song));
		update_song_number(playlist_song);
		}
	update_song_count(playlist_count);
	redraw_display();
}

static void remove_all_button_pressed()
{
	if (playlist == 0) return;
	playlist_clear();
	gtk_clist_clear(GTK_CLIST(playlist_clist));

	/* if playing STOP */
	if (status == STATUS_PLAY)
		{
		stop_playing_file();
		status = STATUS_STOP;
		}
	if (status == STATUS_PAUSE) status = STATUS_STOP;

	/* notify user playlist is empty */
	playlist_song = -1;
	set_song_info(playlist_song);
	redraw_name(get_song_title(playlist_song));
	update_song_number(playlist_song);
	update_song_count(playlist_count);
	redraw_display();
}

static void up_button_pressed()
{
	gchar *buf[] = { "x", };

	if (playlist_pos < 1) return;
	playlist_swap_items(playlist_pos - 1, playlist_pos);

	gtk_clist_freeze (GTK_CLIST (playlist_clist));

	buf[0] = playlist_get_title(playlist_pos - 1);
	gtk_clist_remove(GTK_CLIST(playlist_clist),playlist_pos);
	gtk_clist_insert(GTK_CLIST(playlist_clist),playlist_pos - 1, buf);

	update_list_highlight(playlist_clist, playlist_pos, playlist_pos - 1, TRUE);
	gtk_clist_thaw (GTK_CLIST (playlist_clist));

	if (playlist_song == playlist_pos)
		{
		playlist_song--;
		update_song_number(playlist_song);
		redraw_display();
		}
	else if (playlist_song == playlist_pos  - 1)
		{
		playlist_song++;
		update_song_number(playlist_song);
		redraw_display();
		}
	playlist_pos--;
}

static void down_button_pressed()
{
	gchar *buf[] = { "x", };

	if (playlist_pos > playlist_count - 2 || playlist_pos < 0) return;
	playlist_swap_items(playlist_pos, playlist_pos + 1);

	gtk_clist_freeze (GTK_CLIST (playlist_clist));

	buf[0] = playlist_get_title(playlist_pos + 1);
	gtk_clist_remove(GTK_CLIST(playlist_clist),playlist_pos);
	gtk_clist_insert(GTK_CLIST(playlist_clist),playlist_pos + 1, buf);

	update_list_highlight(playlist_clist, playlist_pos, playlist_pos + 1, TRUE);
	gtk_clist_thaw (GTK_CLIST (playlist_clist));

	if (playlist_song == playlist_pos)
		{
		playlist_song++;
		update_song_number(playlist_song);
		redraw_display();
		}
	else if (playlist_song == playlist_pos  + 1)
		{
		playlist_song--;
		update_song_number(playlist_song);
		redraw_display();
		}
	playlist_pos++;
}

static void top_button_pressed()
{
	gchar *buf[] = { "x", };
	gchar *data;

	if (playlist_pos < 1) return;

	data = g_strdup(playlist_get_item(playlist_pos));
	playlist_remove_item(playlist_pos);
	playlist_insert_item(data, 0);

	g_free(data);

	gtk_clist_freeze (GTK_CLIST (playlist_clist));

	buf[0] = playlist_get_title(0);
	gtk_clist_remove(GTK_CLIST(playlist_clist), playlist_pos);
	gtk_clist_insert(GTK_CLIST(playlist_clist), 0, buf);

	update_list_highlight(playlist_clist, playlist_pos, 0, TRUE);
	gtk_clist_thaw (GTK_CLIST (playlist_clist));

	if (playlist_song == playlist_pos)
		{
		playlist_song = 0;
		update_song_number(playlist_song);
		redraw_display();
		}
	else if (playlist_song < playlist_pos)
		{
		playlist_song++;
		update_song_number(playlist_song);
		redraw_display();
		}
	playlist_pos = 0;
}

static void end_button_pressed()
{
	gchar *buf[] = { "x", };
	gchar *data;

	if (playlist_pos < 0 || playlist_pos >= playlist_count - 1) return;

	data = g_strdup(playlist_get_item(playlist_pos));
	playlist_remove_item(playlist_pos);
	playlist_add_item(data);

	g_free(data);

	gtk_clist_freeze (GTK_CLIST (playlist_clist));

	buf[0] = playlist_get_title(playlist_count - 1);
	gtk_clist_remove(GTK_CLIST(playlist_clist), playlist_pos);
	gtk_clist_append(GTK_CLIST(playlist_clist), buf);

	update_list_highlight(playlist_clist, playlist_pos, playlist_count - 1, TRUE);
	gtk_clist_thaw (GTK_CLIST (playlist_clist));

	if (playlist_song == playlist_pos)
		{
		playlist_song = playlist_count - 1;
		update_song_number(playlist_song);
		redraw_display();
		}
	else if (playlist_song > playlist_pos)
		{
		playlist_song--;
		update_song_number(playlist_song);
		redraw_display();
		}
	playlist_pos = playlist_count - 1;
}

void add_play_button_pressed()
{
	add_button_pressed();
	start_playing_song_number(playlist_count - 1);
}

void pos_play_button_pressed()
{
	start_playing_song_number(playlist_pos);
}

void playlist_songinfo_pressed()
{
	view_song_info(playlist_get_item(playlist_pos));
}

void filelist_songinfo_pressed()
{
	view_song_info(current_path);
}

void playlist_move_pressed()
{
	file_move_dialog(playlist_get_item(playlist_pos));
}

void filelist_move_pressed()
{
	file_move_dialog(current_path);
}

void playlist_copy_pressed()
{
	file_copy_dialog(playlist_get_item(playlist_pos));
}

void filelist_copy_pressed()
{
	file_copy_dialog(current_path);
}

void playlist_rename_pressed()
{
	file_rename_dialog(playlist_get_item(playlist_pos));
}

void filelist_rename_pressed()
{
	file_rename_dialog(current_path);
}

void playlist_delete_pressed()
{
	file_delete_dialog(playlist_get_item(playlist_pos));
}

void filelist_delete_pressed()
{
	file_delete_dialog(current_path);
}

static void path_entry_select(gchar *newdir, gpointer data)
{
	if (isdir(newdir))
		{
		set_current_dir(newdir);
		refresh_dir();
		return;
		}
	else if (isfile(newdir))
		{
		gchar *buf;
		gchar *ptr;
		gchar *file;
		GList *list;

		buf = g_strdup(newdir);
		ptr = filename_from_path(buf);
		file = g_strdup(ptr);

		if (ptr > buf) ptr--;
		if (ptr == buf)
			{
			g_free(buf);
			buf = g_strdup("/");
			}
		else
			{
			ptr[0] = '\0';
			}
		if (strcmp(buf, current_dir) != 0)
			{
			set_current_dir(buf);
			refresh_dir();
			}
		else
			{
			gtk_entry_set_text(GTK_ENTRY(pathentry), current_dir);
			}

		list = filelist;
		while(list)
			{
			gchar *name = list->data;
			if (strcmp(name, file) == 0)
				{
				g_free(buf);
				buf = g_strconcat(current_dir, "/", name, NULL);
				playlist_add_song(buf);
				g_free(buf);
				g_free(file);
				return;
				}
			list = list->next;
			}

                g_free(buf);
                g_free(file);
		}
}

static void history_menu_select(GtkWidget *widget, gchar *newdir)
{
	set_current_dir(newdir);
	refresh_dir();
}

static gchar *truncate_hist_text(gchar *t, gint l)
{
	gchar *tp;
	gchar *tbuf;
	if (l >= strlen(t)) return g_strdup(t);
	tp = t + strlen(t) - l;
	while (tp[0] != '/' && tp < t + strlen(t)) tp++;
		/* this checks to see if directory name is longer than l, if so */
		/* reset the length of name to l, it's better to have a partial */
		/* name than no name at all. */
	if (tp >= t + strlen(t)) tp = t + strlen(t) - l;
	tbuf = g_strconcat("/...", tp, NULL);
	return tbuf;
}

static void update_history_menu()
{
	static GList *history_list = NULL;
	GtkWidget *historymenu;
	GtkWidget *button;
	gchar *hist;
	gchar *hist_ptr;

	if (history_list)
		{
		g_list_foreach(history_list, (GFunc) g_free, NULL);
		g_list_free(history_list);
		history_list = NULL;
		}

	hist = g_strdup(current_dir);
	hist_ptr = hist + strlen(hist) - 1 ;
	
	historymenu = gtk_menu_new();
	while (hist_ptr > hist)
		{
		gchar *path;
		gchar *hist_trunc;
		hist_trunc = truncate_hist_text(hist,32);
		button = gtk_menu_item_new_with_label (hist_trunc);
		g_free(hist_trunc);

		path = g_strdup(hist);
		history_list = g_list_append(history_list, path);
		gtk_signal_connect (GTK_OBJECT (button), "activate",
			(GtkSignalFunc) history_menu_select, path);

		gtk_menu_append (GTK_MENU (historymenu),button);
		gtk_widget_show (button);

		while (hist_ptr[0] != '/') hist_ptr--;
		hist_ptr[0] = '\0';
		}
	g_free(hist);
	/* looks like we have to add the root directory manually */
	button = gtk_menu_item_new_with_label ("/");

	gtk_signal_connect (GTK_OBJECT (button), "activate",
		(GtkSignalFunc) history_menu_select, "/");

	gtk_menu_append (GTK_MENU (historymenu),button);
	gtk_widget_show (button);

	gtk_option_menu_set_menu(GTK_OPTION_MENU(historypulldown),historymenu);
	gtk_entry_set_text(GTK_ENTRY(pathentry), current_dir);
}

void refresh_dir()
{
	int i;

	if (playlist_window)
		{
		gtk_clist_freeze (GTK_CLIST (dirlist_clist));
		gtk_clist_clear (GTK_CLIST (dirlist_clist));
		gtk_clist_freeze (GTK_CLIST (filelist_clist));
		gtk_clist_clear (GTK_CLIST (filelist_clist));
		}

	read_dir();

	dirlist_sort();
	filelist_sort();

	if (!playlist_window) return;

	for (i=0; i < dirlist_count; i++)
		{
		gchar *buf[] = { "x", };
		buf[0] = dirlist_get_item(i);
		gtk_clist_append(GTK_CLIST(dirlist_clist), buf);
		}
	
	for (i=0; i < filelist_count; i++)
		{
		gchar *buf[] = { "x", };
		buf[0] = filelist_get_item(i);
		gtk_clist_append(GTK_CLIST(filelist_clist), buf);
		}

	gtk_clist_thaw (GTK_CLIST (dirlist_clist));
	gtk_clist_thaw (GTK_CLIST (filelist_clist));

	update_history_menu();
}

static void populate_playlist_clist()
{
	int i;

	gtk_clist_freeze (GTK_CLIST (playlist_clist));
	gtk_clist_clear (GTK_CLIST (playlist_clist));
	for (i=0; i < playlist_count; i++)
		{
		gchar *buf[] = { "x", };

		buf[0] = playlist_get_title(i);
		gtk_clist_append(GTK_CLIST(playlist_clist),buf);
		}
	gtk_clist_thaw (GTK_CLIST (playlist_clist));
}

void show_playlist_length()
{
	if (!playlist_window)
		{
		return;
		}
	else if (playlist_accounting)
		{
		gchar *buf;
		gchar tm[64];
		gint hrs = playlist_length / 3600;
		gint min = (playlist_length - (hrs * 3600)) / 60;
		gint sec = playlist_length - (hrs * 3600) - (min * 60);
		sprintf(tm, "%02d:%02d:%02d", hrs, min, sec);
		buf = g_strconcat("Playlist length: ", tm, NULL);
		gtk_label_set(GTK_LABEL(fileinfolabel), buf);
		g_free(buf);
		}
	else
		{
		gtk_label_set(GTK_LABEL(fileinfolabel), "header loading is off");
		}

}

void show_playlist()
{
	int i;
	if (playlist_window)
		{
		gdk_window_raise(playlist_window->window); 
		return;
		}

	create_playlist_window();

	gtk_clist_freeze (GTK_CLIST (dirlist_clist));
	gtk_clist_freeze (GTK_CLIST (filelist_clist));

	for (i=0; i < dirlist_count; i++)
		{
		gchar *buf[] = { "x", };
		buf[0] = dirlist_get_item(i);
		gtk_clist_append(GTK_CLIST(dirlist_clist), buf);
		}
	
	for (i=0; i < filelist_count; i++)
		{
		gchar *buf[] = { "x", };
		buf[0] = filelist_get_item(i);
		gtk_clist_append(GTK_CLIST(filelist_clist), buf);
		}

	gtk_clist_thaw (GTK_CLIST (dirlist_clist));
	gtk_clist_thaw (GTK_CLIST (filelist_clist));

	update_history_menu();

	populate_playlist_clist();
}

static void hide_playlist()
{
	gtk_clist_clear(GTK_CLIST(dirlist_clist));
	gtk_clist_clear(GTK_CLIST(filelist_clist));
	gtk_widget_destroy(playlist_window);
	playlist_window = NULL;
	playlist_pos = -1;
}

static void create_playlist_window()
{
	GtkWidget *frame;
	GtkWidget *hbox;
	GtkWidget *hbox1;
	GtkWidget *vbox;
	GtkWidget *vbox1;
	GtkWidget *vbox2;
	GtkWidget *button;
	GtkWidget *infopackbox;
#ifdef GTK_HAVE_FEATURES_1_1_4
	GtkWidget *scrolled;
#endif

	gchar *dir_title [] = { "Directories:", };
	gchar *file_title [] = { "Files:", };
	gchar *play_title [] = { "Playlist:", };
	gchar sbuf[255];


	/* set up playlist window */
	playlist_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_policy(GTK_WINDOW(playlist_window), TRUE, TRUE, FALSE);
	gtk_signal_connect (GTK_OBJECT (playlist_window), "delete_event",(GtkSignalFunc) hide_playlist, NULL);
	sprintf(sbuf, "GQmpeg Playlist - %s", playlist_filename);
	gtk_window_set_title (GTK_WINDOW (playlist_window), sbuf);
	gtk_container_border_width (GTK_CONTAINER (playlist_window), 0);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (playlist_window), vbox);
	gtk_widget_show (vbox);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
	gtk_widget_show (hbox);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), vbox1, TRUE, TRUE, 0);
	gtk_widget_show (vbox1);

        /* path entry */
        pathentry = gtk_entry_new_with_max_length(255);
        tab_completion_add_to_entry(pathentry, path_entry_select, NULL);
        gtk_box_pack_start (GTK_BOX (vbox1), pathentry, FALSE, FALSE,0);
        gtk_widget_show (pathentry);

	/* history button */
	historypulldown = gtk_option_menu_new ();
	gtk_box_pack_start (GTK_BOX (vbox1), historypulldown, FALSE, FALSE,0);
	gtk_widget_show (historypulldown);

#ifdef GTK_HAVE_FEATURES_1_1_4
	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(vbox1), scrolled, TRUE, TRUE, 0);
	gtk_widget_show(scrolled);
#endif

	/* directory list */
	dirlist_clist=gtk_clist_new_with_titles (1, dir_title);
	gtk_signal_connect (GTK_OBJECT (dirlist_clist), "button_press_event",(GtkSignalFunc) change_dir, NULL);
	gtk_widget_set_usize (dirlist_clist, list_width, -1);
	gtk_clist_column_titles_passive (GTK_CLIST (dirlist_clist)); 
#ifdef GTK_HAVE_FEATURES_1_1_4
	gtk_container_add (GTK_CONTAINER (scrolled), dirlist_clist);
#else
	gtk_clist_set_policy (GTK_CLIST (dirlist_clist), GTK_POLICY_ALWAYS, GTK_POLICY_AUTOMATIC); 
	gtk_box_pack_start (GTK_BOX (vbox1), dirlist_clist, TRUE, TRUE, 0);
#endif
	gtk_widget_show (dirlist_clist);

#ifdef GTK_HAVE_FEATURES_1_1_4
	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(vbox1),scrolled,TRUE,TRUE,0);
	gtk_widget_show(scrolled);
#endif

	/* file list */
	filelist_clist=gtk_clist_new_with_titles (1, file_title);
	gtk_signal_connect (GTK_OBJECT (filelist_clist), "button_press_event",(GtkSignalFunc) change_file, NULL);
	gtk_widget_set_usize (filelist_clist, list_width, -1);
	gtk_clist_column_titles_passive (GTK_CLIST (filelist_clist)); 
#ifdef GTK_HAVE_FEATURES_1_1_4
	gtk_container_add (GTK_CONTAINER (scrolled), filelist_clist);
#else
	gtk_clist_set_policy (GTK_CLIST (filelist_clist), GTK_POLICY_ALWAYS, GTK_POLICY_AUTOMATIC); 
	gtk_box_pack_start (GTK_BOX (vbox1), filelist_clist, TRUE, TRUE, 0);
#endif
	gtk_widget_show (filelist_clist);

	/* button vbox */

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (vbox1), 5);
	gtk_box_pack_start (GTK_BOX (hbox), vbox1, FALSE, FALSE, 0);
	gtk_widget_show (vbox1);

	hbox1 = gtk_hbox_new (TRUE, 5);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 5);
	gtk_widget_show(hbox1);

	button = gtk_button_new_with_label("Home");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) change_dir_to_home, NULL);
	gtk_box_pack_start (GTK_BOX (hbox1), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Close");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) hide_playlist, NULL);
	gtk_box_pack_start (GTK_BOX (hbox1), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	frame = gtk_frame_new("Playlist");
	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (vbox2), 5);
	gtk_container_add (GTK_CONTAINER (frame),vbox2);
	gtk_widget_show (vbox2);

	button = gtk_button_new_with_label("Save");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) save_pressed, NULL);
	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Save as...");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) save_as_pressed, NULL);
	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	button = gtk_check_button_new_with_label("save mode");
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), save_mode_in_playlist);
	gtk_signal_connect (GTK_OBJECT (button), "toggled",(GtkSignalFunc) save_mode_changed, NULL);
	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	button = gtk_hseparator_new();
	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 5);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Load");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) load_button_pressed, NULL);
	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Append");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) append_button_pressed, NULL);
	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	frame = gtk_frame_new("Move Song");
	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (hbox1), 5);
	gtk_container_add (GTK_CONTAINER (frame),hbox1);
	gtk_widget_show(hbox1);

	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox1), vbox2, TRUE, TRUE, 0);
	gtk_widget_show (vbox2);

	button = gtk_button_new_with_label("Top");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) top_button_pressed, NULL);
	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("End");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) end_button_pressed, NULL);
	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox1), vbox2, TRUE, TRUE, 0);
	gtk_widget_show (vbox2);

	button = gtk_button_new_with_label("Up");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) up_button_pressed, NULL);
	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Down");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) down_button_pressed, NULL);
	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	frame = gtk_frame_new("Songs");
	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (vbox2), 5);
	gtk_container_add (GTK_CONTAINER (frame),vbox2);
	gtk_widget_show (vbox2);

	button = gtk_button_new_with_label("Add All");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) add_all_button_pressed, NULL);
	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Add");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) add_button_pressed, NULL);
	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Insert");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) insert_button_pressed, NULL);
	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Remove");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) remove_button_pressed, NULL);
	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Remove All");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) remove_all_button_pressed, NULL);
	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

#ifdef GTK_HAVE_FEATURES_1_1_4
	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(hbox),scrolled,TRUE,TRUE,0);
	gtk_widget_show(scrolled);
#endif

	/* play list */
	playlist_clist=gtk_clist_new_with_titles (1, play_title);
	gtk_signal_connect (GTK_OBJECT (playlist_clist), "button_press_event",(GtkSignalFunc) playlist_item_selected, NULL);
	gtk_widget_set_usize (playlist_clist, list_width, -1);
	gtk_clist_column_titles_passive (GTK_CLIST (playlist_clist)); 

#ifdef GTK_HAVE_FEATURES_1_1_4
	gtk_container_add (GTK_CONTAINER (scrolled), playlist_clist);
#else
	gtk_clist_set_policy (GTK_CLIST (playlist_clist), GTK_POLICY_ALWAYS, GTK_POLICY_AUTOMATIC); 
	gtk_box_pack_start (GTK_BOX (hbox), playlist_clist, TRUE, TRUE, 0);
#endif
	gtk_widget_show (playlist_clist);

	/* create info pack box */
	infopackbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_end (GTK_BOX (vbox), infopackbox, FALSE, FALSE, 0);
	gtk_widget_show (infopackbox);

	/* file info window */
	button = gtk_frame_new (NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(button),GTK_SHADOW_IN);
	gtk_box_pack_end (GTK_BOX (infopackbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);

	fileinfolabel = gtk_label_new ("");
	gtk_container_add (GTK_CONTAINER (button), fileinfolabel);
	gtk_widget_show (fileinfolabel);
	show_playlist_length();

	gtk_widget_show(playlist_window);

	dnd_set_drop(playlist_clist, GTK_CLIST(playlist_clist)->clist_window);
}


